#<20170328.1512.53>
# AddConf does most of the writting of the configure file with all the comments.
#
# parameters:
#    fid      The file to 'put' to
#    var_var  The name of the variable we are to write instructions to set.
#             If var_var is a list e.g. {config menu,*} it is used to get 
#             an array of entries to set up. In this case, all config(menu,*)
#             entries.
#    setto    is a flag with values 
#             ""  -> single line set statment
#             1   -> print one line per each level entry in the list
#             2   -> print one line per first rest on second line
#             3 -> print 'set var {' caller to complete...
#    expl     This is the comment to be printed prior to the set statment.
#             If not empty, a line of "-----" is first printed to seperate 
#             the text from the prior entry in the file.
#
proc AddConf { fid var_var setto expl } {
  global glob config 
  if { $expl != "" } {
    putsTrim $fid {`{fill -}`}
    putsTrim $fid $expl 
  }
  # Check for something like {config menu,*}
  lassign $var_var arrayname pattern 
  set r [catch {array get $arrayname $pattern} varList]
  if {$r == 0 && $varList !={}} {
    # Yep, break that up...
    set var_var {}
    foreach {ent dum} $varList {
      lappend var_var ${arrayname}($ent)
    }
  }
#    puts "$var_var"
  foreach val $var_var {
    #  upvar $var_var var
    upvar $val var
    AddConfPutVar
  }
}


proc AddConfPutVar {} {
  uplevel {
    puts  -nonewline $fid "set $val"
    if { $setto == "" } {
      if {[string match "* *" $var] || $var == {}} {
	puts $fid " \{$var\}"
      } else {
	puts $fid " $var"
      }
    } elseif { $setto == "1" || $setto == "2"} {
      # make single line per first level entry
      puts $fid " \{"
      foreach kk $var {
	puts -nonewline $fid "    "
	if {$setto == "2"} {
	  if {[string match "* *" $kk] || $kk == {} } {
	    puts $fid "\{$kk\}"
	  } else {
	    puts $fid "$kk"
	  }
	} else {
	  if {[set t [llength $kk]] != 1} {
	    puts -nonewline $fid "\{"
	  }
	  set in 0
	  foreach k $kk {
	    if {$in > 0} {
	      puts -nonewline $fid "\n        "
	    } 
	    if {[string match "* *" $k] || $k == {} } {
	      puts -nonewline $fid "\{$k\}"
	    } else {
	      puts -nonewline $fid "$k"
	    }
	    incr in
	  }
	  if {$t != 1} {
	    puts $fid "\}"
	  } else {
	    puts $fid ""
	  }
	}
      }
      puts $fid "  \}\n"
    } else {
      puts -nonewline $fid " \{"
    }
    puts $fid ""
  }
}

proc ClearTheDecksHereComeANewConfig {} {
    global glob config
    # A call to this proc is in the front end of the config file.
    # What we need to do is eliminate any old stuff that it may not
    # overlay.  Mostly this will be array entries that have been eliminated.

    # Eliminate the old bind array so we don't pop it up after the user tried
    # to deep six it....
    # We don't do this for the menu array as it will not bite her/him if it 
    # comes back.  And, of course, changeing it will overlay what is.
    foreach {but val} [array get config "bind,*"] {
	unset config($but)
    }
}
    
proc SaveConfig {}  {
  global glob config env
  lassign [getOldNewVersions] oldv newv
  set r [catch {open $glob(configFile) w} fid]
  if {$r != 0} { PopError "Saving config: $fid"; return "" }

  putsTrim $fid  \
      "# Generated by FileRunner $glob(version) on [clock format [clock seconds]]"

  putsTrim $fid {
    # This is the configuration file for FileRunner. It is read every time
    # FileRunner starts up. It is saved in a sub `{ifThis {$::MSW} {folder of  
    # your HOME foulder, usually in "%%APPDATA%%/.fr". 
    # %%APPDATA%% is the MS windows environment variable APPDATA. (You may
    # see the contents of this variable in a filerunner command window by typing
    # "%set env(APPDATA)" or "echo %%APPDATA%%".)  FileRunner will also recognize
    # these possible locations for the configuration file: "%%LOCALAPPDATA%%/.fr",
    # "%%USERPROFILE%%/.fr"} {directory of your HOME directory, usually 
    # "~/.config/.fr" or "~/.fr "(~ is short for the path to your home directory).}}`
    # The configuration file is named "config<date>" where <date> is the release
    # date of the FileRunner.  New versions of FileRunner will use old config
    # files to build a new one with the new date. This allows old FileRunner
    # versions to continue to use their config files with out conflict with newer
    # versions.
    #
    # Make
    # sure to only edit values for variables that are listed. Do not add or
    # delete comments or other variables, since this file will be automatically
    # regenerated
    # when you do a "Edit Configuration->Save Configuration" in the menu and those
    # changes will be lost. 
    #
    # This config file is saved to disk and reread into FileRunner when you
    # press the OK button in the config browser.

    # You can also edit this file
    # with a regular text editor and re-read it into FileRunner with
    # "Configuration->Reread Configuration" or just restart FileRunner.
    # (If you have the save on exit option set you should exit FileRunner by
    # typing "%exit" in a command window to avoid the auto save messing with
    # your newly edited file.)

    # From time to time, as FileRunner is enhanced, the 'as shipped' version
    # of a configuration entry may change. If you have an existing configuration
    # file, it will replace the 'as shipped' entry with what you have entered
    # or what was last shipped. To reset an entry to the current 'as shipped'
    # version all you need to do is to eliminate that entry from the configuration,
    # either with the configuration editor or another editor. That is all
    # that needs to be done. FileRunner will replace the missing entry with the
    # current 'as shipped' version.
    
    #`{ifThis {$::MSW} {# FileRunner handles file path names as
    # "/" separated components and not "\\" separated as windows uses. In passed
    # versions of FileRunner it was recommended that you use [file native] to
    # pass native MS windows file names. This version ALWAYS passes native
    # file names so that option is no longer needed and in fact will be
    # automaticly removed from the config file.} {}}`

    # Syntax note: Filerunner uses a special Exec Call Formatter (FRECF) to
    # set up calls to external
    # (and some internal) programs and functions. To get details on how the
    # lines in this file are handled please read the "Configure
    # options for commands I" and II in the Tips file (see Help menu).
    
  }
  
  puts $fid "ClearTheDecksHereComeANewConfig"
  putsTrim $fid {
    #**************** HOTLISTS `{fill *}`
  }

  AddConf $fid config(hotlist) 2 {
    # This list is what will show up in your HotList menu. (Like "bookmarks"
    # in a browser). Feel free to edit this list.
    # You can label your selections thusly: {label /home/usr/foobar} 
    # and you can build as many levels of cascade menus as you like
    # with each cascade starting with an entry preceeded by "-"
    # thusly:`{next;in 1}`
    # one level {-clabel { a b c }}`{next}`
    #         two level {first {a b {-second {c d e}}}}
    
    # or, more readably:`{next;in 0}`
    
    #         set config(hotlist) {`{next;in 1}`
    #             {-first {`{next;in 1;in 1}`
    #                a`{next}`
    #                b`{next}`
    #                {-second {`{next;in 1;in 1}`
    #                    c`{next}`
    #                    d`{next}`
    #                    e`{next}`
    #                   }`{next;in 0}`
    #                }`{next;in 0}`
    #              }`{next;in 0}`
    #            }`{next;in 0}`
    #          }`{next;in 0}`
    # where each 'a' 'b' 'c' 'd' 'e' is an entry which also may have a label as above.
    # For example here is an entry for the Windows volumes:`{next}`
    # {-volumes {{floppy a:/} {{system volume} c:/} { d:/ }}}`{next}`
    # The '{}' around an entry allow spaces which otherwise seperate the entries.
    # White space is preserved so what ever indention format you use is kept.
    # Try it...
  }

  AddConf $fid config(runlist) 2 {
    # This list is used by the Run ... commands (Utilities menu).  It contains
    # a list of programs that you can select and instructions on how to run them.
    # You may find it easier to add to this list by invoking the Run command
    # and selecting '<Add a new program>' where you can use command completion and
    # or browsing to find the program name.
    #
    # Each entry in the list contains 2 to 5 entries as follows:
    #
    # The first entry is the display name (or 'nice' name) and the run string. 
    # You may choose any thing you want for the display name.  The run string
    # should contain the program name (possibly a full path name) followed by any
    # flags you want to code.  The file(s) you are passing will replace the '%s'
    # in this string. If the program name contains spaces you should can either
    # enclose the full program name in curly braces ({}) or precede each space
    # with a back slash (\). `{ifThis {$::MSW} {Use '/' instead of '\'
    # in file names.}}`
    #
    # The second entry is the string you want to separate the files with 
    # (should you ever pass more than one file).  Usually this is a space, coded as
    # { }.
    #
    # The remaining two entries are 'async' 'fullname' and 'last+current'
    # coded in any order.
    # These are optional entries.

    # If 'async' is present the program is run
    # asynchronously.`{next}` If 'fullname' is coded only full filenames are passed.
    # `{next}` If 'last+current' is coded both the last & the currently selected
    # file names are passed.
    
    # Here is an example entry to run the 'thunderbird' mailer to compose a 
    # mail message with the selected files as attachments. It will display as
    # 'mail-to' when the 'Run' command is invoked.
    #
    #  {{mail-to {thunderbird -compose attachment='%r'}} , fullname async}
    #
    # Here is how to call emacs to diff two files:
    #
    #  {{Emacs-diff {emacsclient -e {(edif-files %s %s )} &}}}
    
    # If you need to use the quotes '"' use '%' to protect them e.g.'%"'.
    # (Note that in the mail-to case the parameter is 'attac....'. Any spaces (except
    # in file names, which are autmatically escaped) will make more than one parameter
    # and (likely) fail.
    #
    # The display of the run list (when the run command is given) may be setup
    # as a cascade (or hierarchical) menu. To do this the contents of each
    # sub menu must be contained in '{-title sub-menu-entries}' (note the "-").
    # For example
    # this would be the entry for two versions of running fr as root, one with
    # the home dir change to root's dir and one with the current home dir: `{next}`
    
    #  {{-root-fr-options} { `{next;in 1;in 1}`
    #              {{{using root's home} {sudo -H fr}} { }}`{next}`
    #              {{{using my     home} {sudo fr}} { }}`{next;in 0}`
    #             }`{next;in 0}`
    #  }
    # White space is preserved so what ever indention format you use is kept.
  }  
  

  putsTrim $fid {
    #****** MOUSE & KEY BINDINGS & MENUS `{fill *}`
  }
  AddConf $fid {} "" {
    # This is the binding list. There are two types of bindings, global and 
    # local to a directory.  Global bindings may be triggered anywhere in
    # filerunner's main window, except that local bindings will suppress global
    # bindings in the directory windows. So, for example mouse 3 can be locally 
    # bound to ViewOne and globally bound to a menu. In a directory window mouse-3
    # will do ViewOne, while outside of the directory it will bring up the menu. 
    # Global bindings look like:

    # config(global-bind,event) action

    # local bindings look like:

    # config(bind,event) action

    # Each entry binds a key or mouse event to an action. 
    # Legal actions are all the possible (not just those you configured
    # to be present) commands in the center button list, including
    # user defined buttons, plus the following commands from other filerunner 
    # menus (listed below). Of these commands 'CmdRunCmd' 'Back' and 
    # 'UpDirTree' take parameters. For 'CmdRunCmd' the parameter is 
    # the 'nice' name of a program that must appear
    # in the runlist (see above).  This parameter is
    # optional. For example, using the 'mail-to' example in the runlist, this:
    #
    # set config(bind,Button-2) {CmdRunCmd mail-to}
    #
    # will cause a mouse button 2 press to run thunderbird to bring up a mail 
    # compose window with the selected files 'attached' to the message.
    # The commands 'Back' and 'UpDirTree' must be coded exactly as shown.
    # 'Back' pops a dir from the dir stack and makes it the current dir 
    # (the same as the <- button).  'UpDirTree' is the same as the ^ button.
    # One of these commands might be bound to the Left key (left arrow).  You
    # then might want to bind the Right key (right arrow) to 'ViewOne'.
    # The 'Toggle' command takes as a parameter a config item and toggles it.
    # For example 'Toggle config(fileshow,all)' All the binary items in the 
    # Configuration menu may be toggled this way.  Binary config options not
    # in this menu will, most likely not work, especially if their changes require
    # rebuilding the window (e.g. the keyboard support binary).
    # Legal values for the 'bind' command may be found in the man page (do a
    # google search for 'tk bind').  In general, mouse buttons 1, 2 & 3 with
    # modifiers 'Double, Triple, Meta, Alt, Control etc. may be used. (Do
    # understand that 'Double' & 'Triple' will also invoke the actions leading
    # to the given action, that is a Triple will also invoke the single and double.
    # Also, the button 1 selection will happen even if button one is also
    # bound to a command (this is not recommended). Also note that all printing
    # chars are used by the position command. If you do bind a printing character
    # that binding as well as the position command will be executed.
    #
    # In the list below, the first element in each entry is the 'nice'
    # label, the  second is the actual command name. You may use either one in
    # bind and menu configuration. This is a list of the bindings filerunner
    # ships with.  These may be changed or but not eliminated at your discretion
    # (since these are predefined they will not go away if you just eliminate them 
    # but you may set them to null "{}" to make them do nothing):

    #   set config(bind,Button-3) ViewOne`{next}`
    #   set config(bind,Double-1) {DoMenu,three}`{next}`
    #   set config(bind,Control-3) ViewDirOpposite`{next}`
    #   set config(bind,Control-1) {DoMenu,sample}`{next}`
    #   set config(bind,Button-2) {DoMenu,two}`{next}`
    #   set config(global-bind,Button-3) {DoMenu,two}

    # Here is the current list:
  }

  foreach k $glob(buttoncmds) {
    if { [lindex $k 1] != "" || $k == {}} {
      putsTrim $fid "#    {$k}"
    } else {
      putsTrim $fid "#    $k"
    }
  }
  putsTrim $fid "\n"
  
  AddConf $fid {config bind,*} "" {}
  
  AddConf $fid {config global-bind,*} "" {}
  
  AddConf $fid {config menu,*} "1" {
    # These are the menus.  Each menu is a list of commands & menu references.
    # a menu reference looks like 'DoMenu,name' where 'name' is the menu's 
    # name and should appear here in a config(menu,name) description.
    # You may define as many menus as you like, however, only menus that
    # are referred to by one of the bindings listed above (and cascade menus
    # of those) will be built. An empty entry (i.e. {}) will generate a
    # separator in the menu.  All of these menus may be 'torn off' by clicking
    # on the top entry (the '-----').  This promotes the menu to a window which
    # may be moved around on the screen.  Torn off menus no longer have access
    # to the list box entry that you were on when the menu was first popped up
    # but they do have access to the current selection(s). To continue our 
    # example of using the run command, here is a menu line that will give a
    # menu entry called 'Mail To' which will run 'thunderbird' (see the runlist 
    # above) to bring up a compose window with the selected files attached.
    #
    #    {{Mail To} {CmdRunCmd 0 mail-to}}
  }
  AddConf $fid config(no_selection) 1 {
    # Commands in this list will not update the selection when they are 
    # selected via a bind command.  All other commands will up date the selection
    # if, when they are called via a 'bind'ing, there is no current selection.
    # You should only add to this list if you really know what you are doing.
    # These commands need to know how to work without a selection.
    # Removing commands from this list will make the removed commands update the
    # selection the same as others do.
  }
  AddConf $fid config(paste) 1 {
    # The "paste" configuration item allow you to define additional mouse or
    # key selections that do a "paste".  The standard paste characters
    # <Control-v> and <Control-y> are already defined.  In addition some systems 
    # define <Button2-ButtonRelease> and <Control-Shift-Insert>.  This info is
    # required for paste operations in the directory line where the paste is
    # intercepted to make magic happen. At the same time, any sequence named 
    # here will result in a 'paste' being preformed (without the magic) in all
    # filerunner 'view' windows.  If your system has other sequences for 'paste'
    # but you don't used them to paste in the directory line you don't need to
    # enter them here.  Filerunner is shipped with <Button2-ButtonRelease> set
    # as this is common in the Unix world. Syntax is the same as the Tc/Tk
    # bind command (see bind(n)).
  }
  putsTrim $fid {
    #****** COMMAND CONFIGURATION *`{fill *}`
  }



  AddConf $fid config(space) "" {
    # The configuration elements below that contain '%s' are passed through
    # a formater which causes the loss of any distinction between a space
    # in a parameter (program name, file name, or what ever) and the space
    # that separates them.  In order to retain this information this variable
    # is used to replace the meaningful spaces prior to the format call and
    # thus recover them after the format call.  In the wildly likely case
    # that I have chosen a favorite character sequence of yours, here is
    # your chance to change it.
    #
    # Please note that you are not required to use this sequence in the 
    # below code, but just to indicate that spaces are meaningful in a
    # character string by enclosing it in curly braces e.g. {hi there}
    #
    # While on the subject of quoting or escaping stings, the following
    # characters, if they appear in one of these stings, must be quoted:
    #  \[ { } $ "  Also, depending on the string use, it may be necessary to
    # quote characters that have special meaning to the shell that the command 
    # is passed to.  Quoting may be by {} as indicated above or by '\'.
    # For example this string: $ID:.*Exp $ (used in my diff command) is quoted:
    # {$ID:.*Exp $} to protect both the $ and the space. You could also use:
    # \$ID:.*Exp\ \$ to get the same result.
  }

  AddConf $fid config(editor) "" {
    # This variable should be set to the name of your favorite text
    # editor. This editor will be invoked when you select files and press
    # the "Edit" button. It is also invoked when you double click on any
    # file name in a list box.  The text editor you choose should be an
    # window application, not a text-terminal application. You can start
    # text-terminal editors for example by launching them from inside an
    # xterm. E.g: "xterm -e vi %s". The editor you choose should be able to
    # load multiple files specified as arguments to the editor (e.g: 
    # "emacs /tmp/foo /home/joe/bar"). The editor will be launched in the
    # background.
    
    # Note: If you use a path name and any elements of it contain blanks
    # you will need to do something like:`{next}`
    # set config(editor) {`{next;in 1 5}`
    # {c:/Program Files/Windows NT/Accessories/wordpad.exe} %s}`{next;in 0}`
    # You could also just put the path in you PATH env var, either in your system
    # environment set up or in the config file in filerunner's home directory:`{next}`
    # `{ifThis {$::MSW} {
    # global env; set env(PATH) "$env(PATH);<path to wordpad program>".} {
      # global env; set env(PATH) "$env(PATH):<path to emacs program>"}}`
  }

  AddConf $fid config(view,extensions) "1" {
    # This parameter contains a list of commands and the file patterns that
    # are used when finding the right viewer for various files. The commands
    # will receive a list of selected files. The first file in the selected
    # list are used when deciding which viewer should be used. (Don't try
    # and view a couple of text files and images simultaneously by selecting
    # and pressing the view button). If you add the option "-viewtext"
    # last in the rule that means the command outputs a listing/view and it
    # should be viewed in the internal text viewer. The viewing commands
    # will be executed in the background. View may be used to launch applications
    # based on the file extension, however, for some files what makes more
    # sense to launch the application in a 'peek' mode.  For example you
    # may want to 'peek' inside a .zip file to see what it contains with:
    
    # { {unzip -lv %s} {*.zip} -viewtext }
    
    # where the '-lv' tells unzip to list the files in the package.
    # This is particularly true of files that are addressed by the UnArc and
    # the UnPack buttons where, for example we might want:
    
    #  { {unzip -oqq %s} {*.zip} }
    
    # to actually unzip the file.
  }

  AddConf $fid config(open,extensions) "1" {
    # This parameter contains a list of commands and the file patterns that
    # are used when finding the right 'open' routine for various files. The
    # commands will receive a list of selected files. The first file in the
    # selected list are used when deciding which opener should be used. (Don't
    # try and open a couple of text files and images simultaneously by selecting
    # and pressing the open button). If you add the option "-viewtext"
    # last in the rule that means the command outputs a listing/view and it
    # should be viewed in the internal text viewer. The opening commands
    # will be executed in the background. Open is used to launch applications
    # based on the file extension, however, for some files it makes more
    # sense to launch the application in a 'peek' mode.  For this see the
    # view command.
  }

  AddConf $fid config(cmd,open) "" {
    # This is command for the 'Open' command (above). `{ifThis {$::MSW} {
    # This should be `{next}`'cmd /c start "x"' The "x" in 'start' syntax is the window
    # title and, if missing causes 'start' to interpret file names (especially quoted
    # ones) as the title.} {I am aware of 'gnome-open' 'kde-open' and 
    # 'xdg-open' (one file limit)  and 'gvfs-open' (runs program for each file 
    # rather than send all files to the same program).}}`
    # `{next}`Mac systems use 'open'. 
  }

  AddConf $fid config(cmd,print) "" {
    # The print command you want to use to print files.
  }

  AddConf $fid config(cmd,term) "" {
    # The name of your favorite terminal program.
    # This is the program that will be launched when you press the button
    # with the image of the terminal window (upper right of the file list
    # panels). It will be launched in the background.  If you include a %s
    # in this command it will be replaced with the current working dir 
    # e.g.:
    
    {konsole --workdir %s}
    
    # The command is also invoked in the current working dir.
  }

  AddConf $fid config(cmd,diff) "" {
    # Your favorite diff command. Should be able to diff directories also.
    # Used in the "Diff" button command. Filerunner will wait
    # until the program finishes unless you put a trailing '&' here.
    # You should do this if your diff program opens its own window e.g meld or
    # emacs. Since you asked, this is what you need here to get emacs to 
    # diff twofiles:`{next}`
    #
    # set config(cmd,diff) {emacsclient -e {(ediff-files %s %s% )} &}
    #
    # If you use a non-gui program leave off the '&' and filerunner will
    # pop up
    # a window with the results.`{ifThis {$::MSW} { For windows
    # you might want to use "fc". }}`
  }

  putsTrim $fid {
    #****** ARCHIVER/PACKER CONFIGURATION `{fill *}`
  }

  AddConf $fid config(cmd,unarc,extensions) "1" {
    # Extensions and bindings for unarchive rules. This is a list of
    # commands and file name extensions. Filerunner will use this variable
    # when deciding which command should be used to unarchive the file when
    # you press the UnArc button. See above comments on the 'view' command.
    # Its OK to put identical rules here and in the following 'unpack'
    # command to, for example, unzip a file even if you push the wrong
    # button.
  }

  AddConf $fid config(cmd,unpack,extensions) "1" {
    # Extensions and bindings for unpacking rules. This is a list of
    # commands and file name extensions. Filerunner will use this variable
    # when deciding which command should be used to unpack the file when you
    # press the UnPack button.
  }

  AddConf $fid config(cmd,pack) "" {
    # The name of your favorite packer command. 
    # Used in the "Arc" button command to pack files. Run synchronously
    # or in the background.
  }

  AddConf $fid config(cmd,archive) "" {
    # This is the name of your archiver program. The archiver program is
    # passed the name of the directory selected and is executed in the
    # current directory. The archive created is supposed to contain the
    # contents of the directory. The default built in "tar+gz %s" creates
    # an archive named <directory>.tar.gz packed with tar and gzip. This
    # command is used in the "Arc" button command to pack/archive
    # directories. It can be run synchronously or in the background.
    # This command takes up to three parameters (%s)'s.  The first is the
    # selected file (or directory) the second is the other panels working
    # directory and the third is again the selected file.  Thus:
    
    # "tar -cz  %s -f %s/%s.tar.gz"

    is the same as "tar+gz %s" except
    # that the result will be put in the 'other' directory.
  }

  putsTrim $fid {
    #****** SFTP/FTP CONFIGURATION `{fill *}`
  }

  AddConf $fid config(ftp,login) 1 {
    # Rule based login information. Allows automatic determination
    # if the login should use a proxy and what username/password it
    # should use etc.
    # See the online guide, chapter 8.8.
    # Each entry in this list consists of the following:
    
    # URL of the connection (including ':port' if not the standard port)`{next}`
    # The user name and 'password info' `{next}`
    # A proxy name.
    
    # Any but the URL may be blank if not needed.
    # The 'password info' has the following three parts:
    
    # A password (may be {} for sftp connections using ssh keys)`{next}`
    # The disk address of the ssh private key (may be a full address or 
    #    the file name if the key is in env(HOME)/.ssh)`{in 1 5;next;in 0}`
    # The pass phrase.`{next}`
    #
    # If any of these entries contain blanks, enclose the whole
    # entry in braces e.g.:`{next}` {Hello world}`{next}`
    # Note that filerunner will encrypt the 'password info'.  If you need to change
    # it you should delete the string:`{next}`
    #'>> list of numbers <<'`{next}`
    # and enter the new
    # info in plain text.  File runner will encrypt it automatically.
  }

  AddConf $fid config(ftp,fastlink) "" {
    # Set this to 1 if you feel your FTP connections are fast, to 0
    # otherwise. If you set it to 1, soft-links in FTP listings are
    # correctly identified either as links to directories or to files,
    # otherwise it will always think links are links to directories.
  }

  AddConf $fid config(ftp,proxy) "" {
    # If you have to go through a firewall via an FTP proxy, set this
    # variable to the hostname of your FTP proxy. Set it to "" if you don't
    # have an FTP proxy.
  }

#  AddConf $fid config(ftp,user) "" {
# Set username for FTP sessions. If you are doing anonymous FTP, set
# this to "anonymous".
#} 

  AddConf $fid config(ftp,password) "" {
    # Set password for anonymous FTP sessions. This should be your 
    # email address.
  }

  AddConf $fid config(ftp,timeout) "" {
    # Set timeout in seconds for FTP communication. 
  }

  AddConf $fid config(ftp,anonymous) "" {
    # 1 -> Do anonymous FTP`{next}`
    # 0 -> Log in to FTP with username and password.
  }

  AddConf $fid config(ftp,useproxy) "" {
    # 1 -> Use defined ftp proxy`{next}`
    # 0 -> Don't use proxy (to access hosts inside the proxy/firewall)
  }

  AddConf $fid config(ftp,cache,maxentries) "" {
    # Maximum number of directory listings the cache should hold.
  }

  AddConf $fid config(ftp,cd_pwd) "" {
    # If true (non-zero): When doing a cd to a new directory, Filerunner
    # will run pwd to check where it ended up (which makes a difference
    # if you cd to a link which points to a directory).
  }
  putsTrim $fid {
    #****** RSYNC CONFIGURATION `{fill *}`
  }

  AddConf $fid config(rsync,table) "2" {
    # Rsync is a program that does a very good job of copying files, especially
    # from one system to another. Filerunner is set up to dispatch "rsync"
    # in such a way that it does the copy directly with the target system
    # using one of the two ways it has of communication with the
    # remote (aka host) system (please see the rsync man page for details).
    # This command assumes that the remote file system is mounted to the
    # current system such that filerunner can display it in one of the panes.
    # In order to do the rsync transfer filerunner needs to translate directory names as
    # they appear in filerunner to the names as they appear to rsync. 
    # Filerunner handles the following connections (with needs given):
    #
    # nfs     password  (translation of diretory name automatic)`{next}`
    # cifs    password  share address on the remote`{next}`
    # ftp     share address on the remote`{next}`
    # sftp    share address on the remote`{next}`
    #
    # The rsync table should contain an entry for each remote machine
    # "share". Each entry contains 5 parts {share path mode ops pwd}.
    #
    # share    is the cifs "share", nfs "host" or (s)ftp host 
    #          (for nfs and cifs this is the same as the fstab entry.
    #          `{in 1 9}`For (s)ftp
    #          this is the same as in the dir line when first opening a connection. 
    #          It might look like "ftp://sunsite.unc.edu" for example.`{next;in 0}`
    # path     is the host's path to the "share"
    #          (for nfs fr finds this in the mount info)`{in 1}`
    #          The path should not end in "/". I.e. if the file system root
    #          is the path, enter {}. Windows paths should be entered with
    #          "/", not "\".`{next;in 0}`
    # mode     the mode the rsync connection should use, 
    #          one of (shell daemon{=module} use=altnateCmd)`{in 1}`
    #          rsync daemon mode supports modules. A module should match a 
    #          share name. The use= option allows you to use a program
    #          other than rsync.  scp is a possible alternative.`{next;in 0}`
    # ops      the option string to pass to rsync for this connection
    #          If spaces appear in this string the whole string should be`{in 1}`
    #          quoted with {} e.g. {-a -v}`{next;in 0}`
    # pwd      the password to use on this connection.
    #          For (s)ftp the default is to use the password used for that`{in 1}` 
    #          connection (which fr already knows).`{next;in 0}`
    #
    # Where to put the port number? Usually the remote shell is ssh. The port
    # should be in the "ops" like this (see man page):
    #
    # --rsh="ssh -p <port>"    where <port> is the port number. (Note quotes)
    #
    # If 'use=scp' the port would be '-P <port>' (Note upper case P)
    #                                          
    # The first entry should have a null share and path and sets the default
    # mode, ops and pwd.  These will be used when other entries don't supply
    # this info. It is not actually required to have other than the default
    # entry for nfs connections. If the shell or use=scp mode is used, the 
    # password may not be required (see man pages on setting up password 
    # free ssh/scp connections).
    #
    # Example:
    #
    # set config(rsync,table) {`{next;in 1 3}`
    #    { {} {} shell {-az} mypwd}`{next}`
    #    {//roadracer/c c:/users/me }       (uses default mode, ops, pwd)`{next}`
    #    {ftp://ftp.yonder.com /home/me/ shell {-az} mypwd}`{next}`
    #    {ftp://ftp.wild.com /home/me/ shell {-az --rsh="ssh -p 1022"} mypwd}`{next}`
    #    {sftp://tame.com:1022 /home/you use=scp {-BC -P 1022}}`{next}`
    #    }`{next;in 0}`
    # Note on the last entry the ':1022' port is not used but must be here
    # to match the sftp dir line. The actual port used is supplied by '-P 1022'
    #
    # Note passwords are entered in plane text and encripted automaticlly see
    # the (s)ftp entry above for more about passwords.
    # 
  }
  putsTrim $fid {
    #****** HTTP CONFIGURATION `{fill *}`
  }

  AddConf $fid config(http,proxy) "" {
    # HTTP proxy. Format: host:port (for example: proxy.foo.com:80). 
    # Leave empty if you have a direct connection to the net.
  }

  putsTrim $fid {
    #****** APPEARANCE CONFIGURATION `{fill *}`
  }

  AddConf $fid config(gray) "" {
    # The color gray is spelled grey in some places.  If you put your favorite
    # spelling here, we promise to use that spelling when offering up color
    # names that contain gray/grey.  Note that all named color values that have 
    # gray in their names are duplicates of those with grey in their names (and
    # vice versa) so we need contend with only one spelling.
  }

  AddConf $fid config(gui,color_bg) "" {
    # Background color of listboxes etc.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_fg) "" {
    # Foreground color of listboxes etc.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_cmd) "" {
    # Color for commands in the command window.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_select_bg) "" {
    # Selection background color.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_select_fg) "" {
    # Selection foreground color.
    # You can edit this easier from the configuration menu.
  }
  AddConf $fid config(gui,color_highlight_bg) "" {
    # Highlight background color. This is the color of the last
    # selection and is the second argument for the diff command.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_highlight_fg) "" {
    # Highlight foreground color.This is the color of the last
    # selection and is the second argument for the diff command.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_handle) "" {
    # The 'handle' color is the color of the 'handles' that allow
    # you to change the size of the listbox columns.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_scheme) "" {
    # Scheme color.
    # Default Tk seems to be "grey85".
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_cursor) "" {
    # Cursor color.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_flash) "" {
    # Flash color is used in command completion messages as 
    # well as in search messages in view windows and the 
    # command windows.

    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,color_balloonHelp_fg) "" {
    # Balloon Help foreground color.
    # You can edit this easier from the configuration menu.
  }
  AddConf $fid config(gui,color_balloonHelp_bg) "" {
    # Balloon Help background color.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(ListBoxColumns,right) "2" {
    # This structure defines the listbox layout.  The three parts are:
    # The window name (never never NEVER change this),
    # the window width,
    # and the window label.
    # You may change the label and the width, but the width is easier to 
    # change by dragging the label separator in the window.
    #
    # Also, the order is the order the sub windows appear.  This you may change,
    # but, again, it is easier to drag the label in the window to do the same thing.
    #
    # Note: the '[_  text ]' below tells the localize routine to replace the
    # 'text' with a local version.  While this should be this way for the disto,
    # feel free to eliminate the [_' and the ']'.
    #
    # You can add and delete entries form the list, however, the first part
    # (the window name) must come from the 'config(ListBoxScripts)' (see below
    # two entries down), so if you add and new script, you can then refer to
    # it in this list.  Also, a given 'window name' may only be used once in
    # each of the list boxes. 
  }
  AddConf $fid config(ListBoxColumns,left) "2" {
    # This is the same as above but for the left listbox.  Usually they will be
    # the same, but that is not required.
  }
  AddConf $fid config(ListBoxScripts) "2" {
    # This is list of scripts that filerunner will use to format column entries
    # in the directory list boxes.  This is here to allow you to add new columns.
    # Each column window is filled with the info provided by the script.  The
    # following variables are available for this script:
    # $file   file name (without the directory part)`{next}`
    # $ffile  file name with the '@' and '/' tags attached`{next}`
    # $dir    directory name`{next}`
    # $type   file type (n normal, d directory, l normal link, ld directory link)
    #         FTP file types are the same as above with a leading 'f'`{in 1 8;next;in 0}`
    # $size   file size`{next}`
    # $mtime, $ctime, $atime The modify, create and access times for the file`{next}`
    # $mode   file mode in wrx format`{next}`
    # $usergroup  files owner ship`{next}`
    # $link   link name or blank`{next}`
    # `{ifThis {$::MSW} {# $nlink  The 
    # short file name if 'DIR' option on (see useDir below)`{next}`} {
    # $nlink  The number of links in the files dir entry`{next}`}}`
    # $inst   either 'left' or 'right', indicates the file list pane`{next}`

    # Please note that these entries are live.  If you mess them up it is possible
    # or even likely that filerunner will not run.  Do keep a back up config file
    # if you plan to tread here.   `{ifThis {$::MSW} {As shipped,
    # there are two entries that will do the 8X3 name, 'short' and 'shortDIR'.
    # The 'short' entry will always work, however, if the 'DIR' option (see useDIR
    # below) is on, the 'shortDIR' is faster. (See the note at the head of this
    # listing for information on how to restore an entry to the 'as shipped'
    # version.)}}`
  }

  AddConf $fid config(columnScroll) "" {
    # This controls the position of the column scroll bar.  Possible values are:
    # 0 No column scroll bar. 
    # 1 Top of the column
    # 3 Bottom of the column
    #  You can also pull the windows left & right with the mouse.
  }
  AddConf $fid config(columnScrollSize) "" {
    # This sets the thickness of the column scroll bar.  Recommended size is 8.
    # The other scroll bars are sized at 11.  This size affects the minimum 
    # column width (must be wide enough to contain the two end arrows).
  }

  AddConf $fid config(gui,ListBoxFont) "" {
    # Font for listboxes etc. Must be fixed-width.
    # A good choice is "TkFixedFont"
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,GuiFont) "" {
    # Font for default Tcl/Tk stuff (menus, buttons etc.)
    # Set this to "TkDefaultFont" in order to not change from what it already is.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(gui,BalloonHelpFont) "" {
    # Font for Balloon Help text
    # Set this to "TkTooltipFont" in order to not change from what it already is.
    # You can edit this easier from the configuration menu.
  }

  AddConf $fid config(geometry,main) "" {
    # The X11 geometry string deciding where and how big the Filerunner
    # main window should be at start up.
  }

  AddConf $fid config(geometry,textviewer) "" {
    # The X11 geometry string deciding where and how big the Filerunner
    # text viewer window should be.
  }

  AddConf $fid config(geometry,qedit) "" {
    # The X11 geometry string deciding where and how big the Filerunner
    # quick-editor window should be.
  }

  AddConf $fid config(middle_button_list) "2" {
    # This is a list of the middle buttons in the order they will be
    # presented.  User defined buttons will be added automatically at the
    # end if they are not already in the list. A button may be disabled by 
    # appending "d" to the button or, if not a user defined button, deleting
    # it or misspelling it.  (Misspelled buttons are quietly spelled
    # correctly and moved to the end of the list with a "d" added.)
    # Feel free to reorder this list.  Syntax is:`{next}`
    # {ButtonName} or ButtonName if on a separate line and no spaces`{next}`
    # or {ButtonName d}          for a disabled button`{next}`
    # or {{Button Name} d}       if there are spaces in the name.
  }

  AddConf $fid config(middle_button_colors) "" {
    # This contains a list of background colors for the middle row command
    # buttons. The syntax is:
    #  set config(middle_button_colors) {
    #    { ButtonName [-]Color }
    #    ...
    #  }
    # An example:
    #  set config(middle_button_colors) {
    #    { Copy #0000ff }
    #    { Delete yellow }
    #    { Delete -red }
    #    { Move -red }
    #  }
    # The Copy button inactive color will be blue and the Delete button 
    # will be yellow. The Move and Delete buttons will turn red only when they
    # are active i.e. when the mouse moves over them. Buttons not mentioned
    # will get the default scheme color.
    #
    # Note: Since choosing an inactive color will automatic choose a lighter 
    #       'active' color, if you want to choose both an inactive and an active 
    #       color for the same button, you should arrange this list to put the
    #       'active' color for the same button after the inactive color for it.
  }

  putsTrim $fid {
    #****** SHELL CONFIGURATION *`{fill *}`
  }

  AddConf $fid config(shell,height,left) "" {
    # Height in lines of text of the left shell text window.
  }

  AddConf $fid config(shell,height,right) "" {
    # Height in lines of text of the right shell text window.
  }

  AddConf $fid config(shell,buffer) "" {
    # Size of shell buffer, in lines of text.
  }

  AddConf $fid config(shell,aliases) "3" {
    # Shell aliases, in the form of a list of doublets:`{next}` " { alias, command } .. "
    # If there are arguments to the command line they are just tagged onto the
    # alias. (I.e, aliases can take arguments)
  }
  foreach k $config(shell,aliases) {
    if { [lindex $k 1] != "" } {
      puts $fid "    {$k}"
    } else {
      puts $fid "    $k"
    }
  }
  puts $fid "  \}\n"

  putsTrim $fid {
    #****** MISC CONFIGURATION `{fill *}`
  }

  AddConf $fid config(startpwd,left) "" {
    # At start up, show this directory in the left panel.
    # If you set it to "", Filerunner will show the working directory when
    # it was invoked. If you put "~/" it will show the home directory.
    # If this is a file Filerunner will show its directory and
    # highlight that file.
  }

  AddConf $fid config(startpwd,right) "" {
    # At start up, show this directory in the right panel
    # If you set it to "", Filerunner will show the  working directory when
    # it was invoked. If you put "~/" it will show the home directory.
  }

  AddConf $fid config(fileshow,all) "" {
    # 1 -> Will show hidden files.`{next}`
    # 0 -> Won't.
  }

  AddConf $fid config(fast_checkboxes) "3" {
    # You can configure check boxes to appear on the top menu bar. The following
    # are available.  You may sort these to get them to appear in your desired 
    # order.  Change the "d" to "Y" to get the box to appear.  The second item
    # in each entry is the value to use as a label for the check box.  Feel
    # free to edit the label or even eliminate it (by using {}).  (Balloon help
    # will still tell you what the box does.) The Help messages appears here to
    # help you understand what each one does. If you move an entry, be sure to
    # move the whole entry, including the Help message. Deleted entries will
    # automatically be added to the end of the list. You may delete all of them
    # to get a fresh list in the "shipped" order.
  }
  set width 0
  foreach ent $config(fast_checkboxes) {
    lassign $ent first second
    set width [expr max($width,[string length "$first $second"]) ]
  }
  foreach ent $config(fast_checkboxes) {
    set mes [lassign $ent first second flag]
    set sp [string repeat " " \
  		[expr {$width - [string length "$first $second"]} ]]
    puts $fid "    \{{$first} {$second} $sp $flag"
    set mes [regsub -all {\n} $mes { }]
    set mesr [regsub -all { +} $mes { }]
    set f 0
    set l "end"
    while {[string length [string range $mesr $f $l]] > $width} {
      set pt [string last " " [string range $mesr $f $l] $width]
      set mesr [string replace $mesr $pt+$f $pt+$f "\n       "]
      #puts "$f  $mesr"
      set f [expr {$f + $pt +7}]
    }
    puts $fid "      $mesr\}"
  }
  puts $fid "  \}\n"


  AddConf $fid config(balloonhelp) "" {
    # 1 -> Will enable balloon help`{next}`
    # 0 -> disables balloon help.
  }

  AddConf $fid config(positiondirs) "" {
    # 1 -> Will 'show' directories when typing a character in to a selected 
    #      directory panel.`{in 1 5;next;in 0}`
    # 0 -> Will 'show' files when typing a character in to a selected 
    #      directory panel.`{in 1 5;next;in 0}`
    # By 'show' is meant that the panel is scrolled so that files (or directories)
    # that start with the typed character will visible.
  }

  AddConf $fid config(focusFollowsMouse) "" {
    # 1 -> Will enable focus following mouse`{next}`
    # 0 -> disables focus following mouse (requires restart of filerunner).
  }

  AddConf $fid config(fileshow,dirs) "" {
    # This variable determines how file listings are sorted. Possible values
    # are  "dirsfirst", "dirslast", "mixed"  This is most easily changed with the 
    # configuration menu.

  }

  AddConf $fid config(fileshow,sort) "" {
    # This variable determines what file listings are sorted on. Possible values
    # are:`{next;in 1 8}` "nameonly", "atime", "ratime", ctime", "rctime", mtime",
    #                 "rmtime", "size" and "extension"`{next;in 0}`
    # atime is access time, ctime is creation time, mtime is modify time
    # the r suffex means reverse.
    # This is most easily changed with the configuration menu.

  }

  AddConf $fid config(sortoption) "" {
    # This variable determines how file listings are sorted. Possible values
    # are "-ascii", "-nocase" and "-dictionary".  This is most easily changed 
    # with the configuration menu. Ascii and nocase should be clear.  Dictionary
    # extends "nocase" such that if two names contain embedded numbers the numbers
    # compare as integers, not characters.   For  example, in dictionary mode, 
    # bigBoy sorts between bigbang and bigboy, and x10y sorts between x9y and x11y.

  }
  
  AddConf $fid config(dateformat) "" {
    # Sets date format. Possible values:`{next}`
    # "yymmdd" and "ddmmyy". (yy=year mm=month dd=day)
  }

  AddConf $fid config(autoupdate) "" {
    # Sets how often Filerunner checks to see if it needs to update its 
    # windows.
    # Set this to 0 to disable. Value is in seconds.
  }

  if {$::tcl_platform(os) == "Linux"} {
    AddConf $fid config(inotify_rate_limit) "" {
      # Sets minimum amount of time between window updates when using inotify
      # (see below).  This allows a trade off of prompt updates vs system load.
      # Value is in seconds.
    }

    AddConf $fid config(inotify_flags) "" {
      # These flags control what
      # events will cause filerunner to update its directory listing(s).  This 
      # update is nearly immediate, however, this requires 'some' overhead.  You
      # can remove some or all of these flags to reduce the overhead. There are
      # two sets of flags, the local ones (inotify_flags) and the non-local ones
      # (inotify_nlflags).  Due to network overhead it is recommended that you
      # use a more restricted (even to nill, i.e. {}) for non-local file systems.
      # This is particularly true for the 'r', 'w', 'a' and 'o' flags.
      # The flags have the following meanings:`{next;in 1 5}`
      #     n  File was created.`{next}`
      #     r  File was read from.`{next}`
      #     w  File was written to.`{next}`
      #     a  File's meta-data (inode or xattr) was changed.`{next}`
      #     C  File was closed (and was open for writing).`{next}`
      #     c  File was closed (and was not open for writing).`{next}`
      #     o  File was opened.`{next}`
      #     m  File was moved away from watch.`{next}`
      #     M  File was moved to watch.`{next}`
      #     d  File was deleted.`{next;in 0}`
      # 'r','w','a','C','c' and 'o' cause just the files line in the window
      #  to be updated while the others require a full reread and display of 
      #  the directory.
    }
    AddConf $fid config(inotify_nlflags) "" {
      # The inotify_nlflags are the same as above but are used on directories
      # that are not local (nfs or cifs).  Due to network overhead you will
      # usually not want to be as aggressive with the Non Local flags.
    }
  }
  
  AddConf $fid config(logsize) "" { 
    # Size of log in bytes. Don't set this too high, or Filerunner might 
    # slow down a little. When the log exceeds this size, it will be 
    # truncated.  
  }

  AddConf $fid config(check_ownership) "" {
    # If this is "1", Filerunner will check as best as it can if you are
    # about to overwrite or delete a file you do not own and warn you about
    # it. This can be handy when working in groups with other people and you
    # want an extra check that you don't accidentally erase someone else's
    # files.  This will slow some operations down slightly.
  }

  AddConf $fid config(keyb_support) "" {
    # If this is 1, it enables short-cut
    # characters for most of the middle buttons which will have their short-cut
    # character underlined.
    
    # In addition (once a directory pane has focus) you can use the keyboard
    # arrow keys and PgUp/PgDown to traversing the list.
    
    # Turning
    # this on eliminates the "jump to files that start with this char"
    # ability when focus is in a directory pane.
  }

  AddConf $fid config(ask,dir_delete) "" {
    # If 1, Filerunner will ask you if you are sure when you delete 
    # directories. If 0, it will go ahead and delete without asking.
  }

  AddConf $fid config(ask,file_delete) "" {
    # If 1, Filerunner will ask you if you are sure when you delete 
    # files. If 0, it will go ahead and delete without asking.
  }

  AddConf $fid config(mwheel,delta) "" {
    # For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
    # number of lines to scroll when turning mouse wheel.
  }

  AddConf $fid config(mwheel,neg) "" {
    # For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
    # mouse button to use for negative scroll movement.
  }

  AddConf $fid config(mwheel,pos) "" {
    # For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
    # mouse button to use for positive scroll movement.
  }

  AddConf $fid config(save_conf_at_exit) "" {
    # If true (non-zero): Filerunner will automatically save configuration and
    # hotlist at exit.
  }

  AddConf $fid config(create_relative_links) "" {
    # If true (non-zero): Filerunner will create relative links
    # instead of absolute links. (e.g ../../foo instead of
    # /a/b/c/foo)`{ifThis {$::MSW} {
    # Windows (NTFS) only allows 
    # absolute links, thus absolute is forced.}}`
  }

  AddConf $fid config(cd_pwd) "" {
    # If true (non-zero): When doing a cd to a new directory, Filerunner
    # will run pwd to check where it ended up (which makes a difference
    # if you cd to a link which points to a directory).
  }

  AddConf $fid config(spellingFilter) "2" {
    # The spelling filter chooses what part of a line is spell checked when
    # you spell check in "View" and "Q-Edit" windows.  In the entries below the
    # filter name is first and is followed by the filter script.  You may add
    # your own filter to this list (see the "Users Guide" for help on this).
    # Several filters are provided and may be configured here.
   
    # Provided filters:
    #
    # {}        This is the default filter and causes the whole line to be checked.
    #           This is the {None {}} entry below.`{in 1 10;next;in 0}`
    #
    # qstrings  This filter can be configured to look for two types of strings:
    #           `{in 1}`Single lines (such as "# line" or "// line") as some
    #           languages use.
    #           And strings that have start and end quote marks which may be
    #           1 to N lines long.  E.g. "string" and "/* string */"
    #
    #           qstrings takes two parameters (as well as the string and line 
    #           number that the spell check code supplies):`{next}`
    #           First, the beginning and ending quote strings, e.g.`{next}`
    #                  {\" \"} or {/* */}`{in 1 5;next;in 0}`
    #           Second, the single line start quote string, e.g.`{next}`
    #                  {#} or {//}`{in 1;next;in 0;in 0}`
    #
    # thisstr   This filter takes, as a parameter, another filter and selects
    #           one of the the strings as the string to check.`{in 1}`
    #           This might be used
    #           if you want to spell check a translation list where the 
    #           original and the translation are two stings on the same line
    #           and you want to only check the first or second string.  
    #           For example {thisstr 1 {qstrings {\" \"} {}}} would check the
    #           second string (use 0 for the first string).
  }

  AddConf $fid config(spellcheck,expect) "" {
    # This Boolean, if 1 forces spell check to use "expect" to interact with
    # the spelling program.  Normally "expect" is required only on Windows 
    # platforms.
  }

  putsTrim $fid {
    #****** DANGEROUS CONFIGURATION `{fill *}`
    # Below this point you can change some of the stuff that is used
    # internally by Filerunner. This should almost never be necessary if the
    # platform is standard Unix or close to standard Unix. If you change any
    # of this below, make very very sure you get it right. (Go read the
    # source!). If you are unsure, don't touch it. If you get it wrong, very
    # bad things can happen...
    # 
    # A note on having the full path to the commands below, like using
    # "/bin/cp" and not just "cp": Lets say you have a dot "." in your path
    # and someone really mean has created a command called "cp" in his home
    # directory that does a "rm" instead. Then you cd to his home directory
    # and type cp <something>, you will invoke his evil manipulated version
    # of cp that removes whatever you intended to copy. The morale of this is
    # to always use full path to critical commands and hope your sysop is
    # not so mean he will rename "/bin/rm" to "/bin/mkdir"...
    # A few commands below do not have specified full path, that's because
    # they aren't very critical and seem to move around between different
    # versions of Unix.
  }
  
  AddConf $fid config(cmd,sh) "" {
    # `{ifThis {$::MSW} {
    # With out CYGWIN you can use 'cmd /C' (which is the default). You can
    # also set this to null (i.e. {}) to use the 
    # filerunner command line handler. This command line handler will run most anything
    # that 'cmd /C' can plus it handles *nix files that start with '#! <file>'
    # as the first line. This depends on <file>s directory being in the %PATH%
    # (but it may be in any directory on the path, not just the one in <file>).
    # This command line handler will also expand wild card characters in the
    # command. The syntax for these is the tcl 'glob' syntax (ask google for
    # the "tcl glob man page").
    # } {	
    # The bourne-shell command. This should almost always be /bin/sh.
    # If an option (such as '-c') is needed to execute a command in the
    # command line, it should be here (for example '/bin/sh -c').}}`
  }

  AddConf $fid config(cmd,cp) "" {
    # The cp (copy-files) command. You use the internal Tcl file copy routine 
    # by making this empty ({}). `{ifThis {! $::MSW} {
    # Otherwise this should almost always be /bin/cp
    # on Unix/Linux systems.
    # The 'pfr' switches should be here (i.e. 'cp -pfr')`{next;in 1 5}`
    # (p->preserve times/owner)`{next}`
    # (f-> force overwrite)`{next}`
    # (r-> if dir, copy recursively)`{next;in 0}`  
    # DON'T put the -i switch to cp here to get Filerunner to display
    # a warning for overwriting a file when copying a file. It won't
    # work and will only make it behave unpredictably.} {
    # Otherwise 'xcopy' works fine.
    # Default is with switches:`{next}`
    #     /I /Q /H /C /K /Y`{next;in 1}`
    
    # (/Q -> Don't display file names)`{next}`
    # (/H -> Copies hidden files)`{next}`
    # (/C -> Continue copying if errors)`{next}`
    # (/K -> Copy attributes)`{next}`
    # (/Y -> Suppresses prompting to confirm...)} }`
    #
  }

  AddConf $fid config(cmd,dircp) "" {
    # This command is used by the 'copy' button when the source is a directory.
    # Again '{}' will use the Tcl file copy code and disallows back-grounding.
    # `{ifThis {! $::MSW} {
    # This can be the same as the 'cp' command above.} {
    # The 'xcopy' command will not copy a single file to
    # a point deeper in the tree if the /S or /E switches are on and we do
    # want one of those on in this command. If you use the '{}' option, when
    # you copy a directory to a new location you will get a new directory at
    # that location with the same name as the source unlike what
    # xcopy does(see below)).`{next}`
    # (/S -> copies directories and sub-directories except empty dirs)`{next}`
    # (/E -> copies directories and sub-directories INCLUDING empty dirs)`{next}`
    # (/I -> Assume destination is a directory)`{next}`
    #
    # Most of (all ?) MS windows copy routines treat the
    # source directory as if the user had added "\*" to it.  This means
    # that the directory pointed at is not copied, but its contents are.
    # Usually (at least from a *nix point of view) one wants the directory
    # replicated in the destination followed by its contents.  If this
    # is what you want, you can tell filerunner to negate the implied
    # "*" by adding "-*" to this configuration parameter as a second
    # entry in the list.  It would then look like:`{next;in 1}`
    #
    # set config(cmd,dircp) {{xcopy /E /I /Q /H /C /K /Y} -*}} }`
    #
  }

  AddConf $fid config(cmd,hardlink) "" {
    # This command is used by the 'HardLnk' button. For information on why you
    # might need this and need it to be recursive please read "File links 101"
    # in the Tips file.`{ifThis {! $::MSW} {
    # The best option is:`{next}`
    # '{{cp -pfrl}}'`{next}`
    # (for meaning of 'pfr' see comments on the 'cp' command above.)`{next}`} {
    # The best option is "{}".  This will then use the internal Filerunner
    # hard link code which recursively call the tcl "file link" command. It is
    # also possible that the GNU version of cp will do the hard link recursively,
    # but this is untested. 'fsutil hardlink create' and 'mklink' will not
    # re-curse over directories.
    # Also, this command wants the parameters in the reverse order which is
    # indicated thusly: {fsutil hardlink create %2@s %1@s}}}`
  }
  if {$::glob(os) != "WIN32"} {
    AddConf $fid config(cmd,chmod) "" {
      # The change mode (chmod) command. This should almost always be /bin/chmod.
    }
    AddConf $fid config(cmd,chown) "" {
      # The change mode (chmod) command. This should almost always be /bin/chown.
    }
  }
  AddConf $fid config(cmd,touch) "" {
    # The touch command. If you have the tcl package "fileutil" you can set this to
    # nill i.e. {}. If you do set it to nill and you don't have the package, fr will
    # tell you about it when you first try to use it.
  }

  AddConf $fid config(cmd,du) "" {
    # The "du" command. Should return size in kilobytes. On many
    # systems other than BSD (?) you need to add a -k argument
    # to this command to get it in kilobytes.
  }

  AddConf $fid config(cmd,find) "" {
    # The find command. If this is {} an internal find routine is used.
    # The internal version is a bit slower (especially for large file sets)
    # and has a bit fewer options but, especially where there is no find
    # command, it does the job.
  }
 AddConf $fid config(cmd,ucmd) "" {
   # This configure builds the 3ed entry in the "Utilities" menu. The first
   # of the two values is the label used in the menu, the second is the command
   # that is evaluated when that entry is selected. 
   # `{ifThis {$::glob(os) != "WIN32"} {As shipped this command
   # runs fr as root, i.e. {{Run fr as root} {exec sudo -H fr &}}.
   # Note that if you don't included the "&" filerunner will wait for the
   # program to exit. You can add what
   # ever you like here if you think some other command is more useful.
   # Or you can enhance what is here by adding full paths and/or additional
   # parameters. (The -H in the default command causes sudo to NOT pass the
   #  current HOME environment, but rather to use root's HOME.)} {The
   # original reason for this command was to provide a way to launch an
   # "elevated" version of filerunner.  The easy way to do this is to create
   # a new link (fr.lnk) for filerunner which has the 'run as administrator'
   # option set. This menu item can then launch filerunner using that
   # link.  On the other hand, you may want to put something here that
   # is more useful to you.}}`

   #  Be careful!
  }

  AddConf $fid config(cmd,rm) "" {
    # By default this is {} (i.e. nil). If so filerunner will use the native
    # Tcl "file delete" command. `{ifThis {$::glob(os) != "WIN32"} {This is here to
    # allow you to configure an alternative to to file deletion, in particular,
    # move to "Trash" or "Recycle Bin". Various *nix systems have their own
    # ways of managing the "Trash".} {I recommend "binit.zip for
    # windows" (ask Google about it).}}`

  }
  AddConf $fid config(cmd,tail) "" {
    # The tail command. This command should take two parameters, the line
    # number of the first line to pass to stdout and the file name.
    # The command should senses writes to the file and passes those lines to 
    # stdout. This is used by the 'follow' option in 'view' windows.
  }
  if { $::tcl_platform(platform) == "windows" } {
    AddConf $fid config(useDIR) "" {
      # This tells filerunner to try different way of gathering dir info,
      # namely by calling the "cmd dir". If 0, the native tcl way is used.
    }
  }
  close $fid
  Log "Configuration saved to $glob(configFile)"
}
#########################################################################
#
#             Fast check boxes
#
#########################################################################
# Each entry consist of a label, the "-command", the variable (may be a 
# list i.e. {variable on-value off-value}) and the balloon help message

# This is the master entry. The first entry in config(fast_checkboxes)
# must match one of these. If not, during set up, that entry will be
# removed from config(fast_checkboxes) and a message sent to the user.
# If one (or more) of these is not in config(fast_checkboxes) it will
# be quietly added at that time.
#
set fast_checkboxes {
  {{Expand Errors} "setupDebug \$glob(debug)" glob(debug) \
       {[_b "Toggles expanded error messages"]}}
  {{Balloon Help} {set ::balloon_help::enable $config(balloonhelp)} \
       config(balloonhelp) \
       {[_b "Toggles the Balloon Help feature (what you are looking at)." ]}}
  {{Position to directories} {} config(positiondirs)\
       {[_b "Toggles the directory vs file positioning for characters typed \
             in a directory panel"]}}
  {{Show All Files} ForceUpdate config(fileshow,all) \
       {[_b "Toggles the show all files flag.\nIf\
            off 'hidden' files are not showen." ] }}
  {{Run Pwd After Cd} {} config(cd_pwd) \
       {[_b "Toggles the do pwd after cd flag" ]}}
  {{Focus Follows Mouse} {after 1 "if {$config(focusFollowsMouse)== 1} \
                         {tk_focusFollowsMouse} "} \
       config(focusFollowsMouse) \
       {[_b "Toggles Focus Follows Mouse flag.\nTakes\
                effect immeadiatly if turned on. \nRequires\
                restart if turned off." ]}}
  {{Run Pwd After Cd (FTP)} {} config(ftp,cd_pwd) \
       {[_b "Toggles the do pwd on FTP directroies after cd flag" ]}}
  {{Anonynomous Ftp}  {} config(ftp,anonymous) \
       {[_b "Toggles the Anonynomous FTP flag.\nIf \
           set FTP login is Anonynomous,\nelse \
           use rule based log in." ]}}
  {{Use FTP Proxy} {} config(ftp,useproxy) \
       {[_b "Toggles the FTP Proxy flag.  If set use Proxy." ]}}
  {{No case on sort} {ForceUpdate} {config(sortoption) -nocase -ascii}\
       {[_b "Sets the sort option to \"no case\".\
             \nOff sets \"ASCII\"." ]}}
  {{Dictionary sort} {ForceUpdate} {config(sortoption) -dictionary -ascii}\
       {[_b "Sets the sort option to \"Dictionary\".\
             \n Off sets \"ASCII\"." ]}}
  {{Sort on aTime} { ForceUpdate} {config(fileshow,sort) atime nameonly}\
       {[_b "Sets the sort option to \"Sort On aTime\".\
             \n Off sets \"Sort on Name\"." ]}}
  {{Sort on Reverse aTime} {ForceUpdate} {config(fileshow,sort) ratime nameonly}\
       {[_b "Sets the sort option to \"Sort On Reverse aTime\".\
             \nOff sets \"Sort on Name\"." ]}}
  {{Sort on cTime} {ForceUpdate} {config(fileshow,sort) ctime nameonly}\
       {[_b "Sets the sort option to \"Sort On cTime\".\
             \n Off sets \"Sort on Name\"." ]}}
  {{Sort on Reverse cTime} {ForceUpdate} {config(fileshow,sort) rctime nameonly}\
       {[_b "Sets the sort option to \"Sort On Reverse cTime\".\
             \nOff sets \"Sort on Name\"." ]}}
  {{Sort on mTime} {ForceUpdate} {config(fileshow,sort) mtime nameonly}\
       {[_b "Sets the sort option to \"Sort On mTime\".\
             \n Off sets \"Sort on Name\"." ]}}
  {{Sort on Reverse mTime} {ForceUpdate} {config(fileshow,sort) rmtime nameonly}\
       {[_b "Sets the sort option to \"Sort On Reverse mTime\".\
             \nOff sets \"Sort on Name\"." ]}}
  {{Sort on Size} {ForceUpdate} {config(fileshow,sort) size nameonly}\
       {[_b "Sets the sort option to \"Sort On Size\".\
             \nOff sets \"Sort on Name\"." ]}}
  {{Sort on Extension} {ForceUpdate} {config(fileshow,sort) extension nameonly}\
       {[_b "Sets the sort option to \"Sort On Extension\".\
             \nOff sets \"Sort on Name\"." ]}}
  {{Ask on File Delete} {} ::config(ask,file_delete) \
        {[_b "On will ask premission to\ndelete files."]}}
  {{Ask on Directory Delete} {} ::config(ask,dir_delete) \
        {[_b "On will ask premission to\ndelete directory trees."]}}
  {{Ask on Delete} {set ::config(ask,dir_delete) \
			[set ::config(ask,file_delete) $::askOnDelVar] } \
       {askOnDelVar 1 0 0} {[_b "On will ask premission to\
             \ndelete files and directories\
             \n(combines the two config options)"]}}
}
  
if  {$tcl_platform(platform) != "windows" } {
  lappend fast_checkboxes \
      {{Create Relative Links} {} config(create_relative_links)\
       {[_b "Toggles the create relative links flag" ]}}
}

proc setDefaultConfig {} {
#  puts "in InitConfig"
  global glob config env argv tcl_platform

  if { $::MSW } {
    set config(hotlist) [list [list -volume [file volume ]]]
  } else {
    set config(hotlist) ""
  }
  set config(runlist) {}
  set config(balloonhelp) 0
  set config(fileshow,all) 0
  set config(focusFollowsMouse) 1
  set config(fileshow,sort) "nameonly"
  set config(fileshow,dirs) "dirsfirst"
  set config(columnScroll) 1
  set config(columnScrollSize) 8
  set config(positiondirs) 0
  set config(startpwd,left) ""
  set config(startpwd,right) ""
  set config(ask,dir_delete) "1"
  set config(ask,file_delete) "0"
  set config(ftp,fastlink) 0
  set config(ftp,proxy) ""
  set config(ftp,user) anonymous
  set config(ftp,password) ""
  set config(ftp,timeout) "120"
  set config(ftp,anonymous) 1
  set config(ftp,useproxy) 1
  set config(ftp,login) {}
  set config(ftp,cache,maxentries) 15
  set config(ftp,cd_pwd) 1
  set config(rsync,table) {{{} {} shell -av {}}}
  set config(http,proxy) {}
  #set config(gui,color_bg) FloralWhite
  set config(gray) "gray"
  set config(gui,color_bg) LemonChiffon
  set config(gui,color_fg) Black
  set config(gui,color_cmd) LightGoldenrod1
  set config(gui,color_select_bg) Yellow
  set config(gui,color_select_fg) black
  set config(gui,color_highlight_bg) white
  set config(gui,color_highlight_fg) red
  set config(gui,color_handle) red  
  set config(gui,color_scheme) "grey85"
  #set config(gui,color_scheme) {#bfeed0add9da}
  set config(gui,color_cursor) black
  set config(gui,color_flash) green
  set config(gui,color_balloonHelp_fg) yellow
  set config(gui,color_balloonHelp_bg) red
  set config(gui,GuiFont) TkDefaultFont

  set config(gui,ListBoxFont) TkFixedFont
  set config(gui,BalloonHelpFont) TkCaptionFont
  
  set config(ListBoxColumns,right) {
    {file 26   {[_ "File/Dir name" ]}}
    {size 7    {[_ "size" ]}}
    {mtime 15  {[_ "mtime" ]}}
    {owner 13  {[_ "owner" ]}} 
    {slink  26 {[_ "link to" ]}}
  }
  if {$::tcl_platform(platform) != "windows" } {
    lappend config(ListBoxColumns,right) \
	{mode 9   {[_ "flags" ]}} \
	{links  3     {[_ "lk" ]}}
  } else {
      set config(useDIR) 1
  }

  set config(ListBoxColumns,left) $config(ListBoxColumns,right)

  set config(middle_button_colors) {
    { Delete -#ff4d4d }
  }
  set config(middle_button_list) {
    Copy 
    CopyAs
    Delete
    Move
    Rename
    MkDir
    S-Link
    S-LnAs
    Chmod
    View
    ViewAsTx
    Edit 
    Q-Edit
    Arc
    UnArc
    UnPack
    ForEach
    Print
    Diff
    {{Rsync copy} {}}
    Select
    HardLnk
    Open
    Run
  }
  # The following gets filled in by the start up script...
  # The start up default is balloon help
  set config(fast_checkboxes) {
    {{Balloon Help} {Tool tips}                                y
      {Toggles the Balloon Help feature (what you are
       looking at).}}
 }
  set config(bind,KeyPress-question) CmdWhatIs
  set config(bind,Button-3) ViewOne
  set config(bind,Double-1) Edit
  set config(bind,Control-3) ViewDirOpposite
  # set config(bind,Control-1) 
  set config(bind,Control-2) {DoMenu,two}
  set config(global-bind,Shift-3) {DoMenu,two}
  #
  # A menu to use with one of the bindings (user can build more)
  #
  set config(menu,sample) [list ViewOne Edit {} menu,two ViewDirOpposite]
  set config(menu,two) [concat {ViewOne} $config(middle_button_list)]
  set config(menu,three) {{{Send To} {CmdRunCmd 0}}}

  set config(no_selection) {ViewOne ViewDirOpposite}

  # These are, at the moment, mutually exclusive...
  #      Sort On Mixed
  #      Sort On Dirs First
  #      Sort On Dirs Last
  # and then
  #        Sort On Time
  #        Sort On Reverse Time
  #        Sort On Size
  #        Sort On Extension
  #        Sort On Name
  set config(paste) {{<Button2-ButtonRelease>}}
  set config(space) {>sp<}
  set config(keyb_support) 0
  set config(dateformat) "yymmdd"
  set config(autoupdate) 20
  set config(inotify_rate_limit) 2
  set config(inotify_flags) "naCmMdw"
  set config(inotify_nlflags) ""
  set config(logsize) 5000
  set config(check_ownership) 0
  set config(geometry,main) "840x720+31+0"
  set config(geometry,textviewer) "80x30"
  set config(geometry,qedit) "80x30"
  set config(spellcheck,expect) 0
  set config(spellingFilter) {
    {None {}}
    {Comment(#&\") {qstrings {\" \"} "#"}}
    {C&C++Comments {qstrings {/* */} "//"}}
  }
  set config(shell,height,left) 10
  set config(shell,height,right) 10
  set config(shell,buffer) 1000
  set config(shell,aliases) {
    {.. {cd ..}} 
    {md mkdir} 
    {m view} 
    {more view} 
    {less view}
  }
  #
  # The helper list...
  set postfix [expr {$glob(cygwin) != {} ? ".exe" : ""}]
  set term xterm$postfix
  set editor "$term$postfix -e vi %s"
  set print "lpr$postfix %s"
  set diff "diff$postfix -r %s %s"
  set pack "gzip$postfix %s"
  set archive "tar+gz %s"

  set config(cmd,sh) "sh -c"
  set config(cmd,cp) "cp -pfr"
  set config(cmd,dircp) "cp -pfr"
  set config(cmd,hardlink) "cp -pfrl"
  set config(cmd,open) "xdg-open"
  set config(editor) "vi %s"

  if { $::MSW } {
    set config(cmd,open) {cmd /c start "started by fr"}
    set config(editor) "{C:/Program Files/Windows NT/Accessories/wordpad.exe} %s"
    if {[info exists env(CYGWIN)] == 0 } {
      # pure windows...
      set config(cmd,sh) "cmd /C"
      set config(cmd,cp) {}
      set config(cmd,dircp) {}
      set config(cmd,hardlink) {{fsutil hardlink create} {r}}
    } else {
      # windows with cygwin (NOT a cygwin version of tcl/tk but rather
      # cygwin is installed...
      # Use the non-windows defs...
    }
  }
  if {[info exists env(EDITOR)]} {
    set config(editor) "$env(EDITOR) %s"
  }
  set config(cmd,chmod) "/bin/chmod$postfix"
  set config(cmd,chown) "/bin/chown$postfix"
  if {$postfix == "" && $tcl_platform(platform) == "windows" } {
    set config(cmd,touch) {}
  } else {
    set config(cmd,touch) "touch$postfix"
  }
  set config(cmd,du) "du$postfix -s"
  set config(cmd,find) find$postfix
  if {$tcl_platform(platform) != "windows" } {
    set config(cmd,ucmd) {
      {Run fr as root}
      {exec sudo -H fr}
    }
  } else {
    set config(cmd,ucmd) {
      {Run fr as admin}
      {exec runas /user:administrator $argv}
    }
  }
  set config(cmd,rm) {}
  set config(cmd,tail) "tail$postfix -f -n +%s %s"
  set config(cmd,print) "lpr$postfix %s"
  set config(cmd,term) "xterm$postfix"
  set config(cmd,diff) "diff$postfix -r %s %s"
  set config(cmd,pack) "gzip$postfix %s"
  set config(cmd,archive) "tar+gz %s"
  
  set config(cmd,unarc,extensions) \
      [list \
	   [list  "gzip$postfix -cd %s | tar xfBp -" \
		{*.tgz *.taz *.tar.z *.tar.gz} ] \
	   [list "tar$postfix xfBp %s"\
		{*.tar} ]\
	   [list "unzip -oqq %s" \
		{*.zip} ]\
	   [list "lha$postfix -x %s" \
		{*.lha} ]\
	  ]
  
  set config(cmd,unpack,extensions) \
      [list \
	   [list "gzip$postfix -d %s" \
		{*.gz} ]\
	  ]
  set config(view,extensions) \
      [list \
	   [list "xmikmod$postfix %s" \
		{*.mod.gz *.s3m *.mtm *.stm *.xm *.uni *.mod} ]\
	   [list "timidity$postfix %s" \
		{*.midi.gz *.midi *.mid.gz *.mid} ]\
	   [list "xv$postfix -gamma 2.0 %s" \
		{ *.bmp *.iff *.jpg *.pbm *.pgm *.png \
		      *.ppm *.tga *.tif *.tiff *.xbm *.xpm *.gif} ]\
	   [list "gzip$postfix -dc %s | ghostview -" \
		{ *.ps.gz } ]\
	   [list "ghostview$postfix %s" \
		{ *.ps } ]\
	   [list "xanim$postfix %s" \
		{*.avi *.fli *.flc *.mpg} ]\
	   [list "gzip$postfix -cd %s | tar tvf -" \
		{ *.tgz *.taz *.tar.z *.tar.gz } \
		-viewtext ]\
	   [list "tar$postfix tvf %s" \
		{*.tar}\
		-viewtext ]\
	   [list "unzip$postfix -lv %s"\
		{*.zip}\
		-viewtext]\
	   [list "lha$postfix -v %s" \
		{*.lha}\
		-viewtext ]\
	   [list "gzip$postfix -cd %s"\
		{*.gz}\
		-viewtext ]\
	   [list "rpm$postfix -qilp %s"\
		{*.rpm}\
		-viewtext ]\
	   ]
  
  set config(open,extensions) {}
  if {$::MSW} {
    set config(open,extensions) [list "msiexec.exe /I %s" "*.msi"]
  }
  
  set config(mwheel,delta) 6
  set config(mwheel,neg) <4>
  set config(mwheel,pos) <5>
  set config(save_conf_at_exit) 1
  
 if {$tcl_platform(platform) != "windows" } {
    set config(create_relative_links) 1
  } else {
    set config(create_relative_links) 0
  }
  
  set config(cd_pwd) 1
  set config(sortoption) "-ascii"
  # Listbox column build scritps.  There should be one for each list box
  # column, by name.
  set config(ListBoxScripts) { 
    {ctime {expr {$ctime=="" ? "" : [GetTimeFromSecs $ctime]}}}
    {atime {expr {$atime=="" ? "" : [GetTimeFromSecs $atime]}}}
    {mtime {GetTimeFromSecs $mtime}}
    {file  {expr {$ffile}}}
    {mode  {expr {$mode}}}
    {owner {expr {$usergroup}}}
    {links {expr {$nlink}}}
    {size  {format %6s $size}}
    {slink {format %3s $link}}
    {blank {expr {""}}}
  }

  if {$::MSW } {
    lappend config(ListBoxScripts) {short {file attributes $file -shortname}} \
	{shortDIR {expr {$nlink}}}
  }
}

proc InitConfig {} {
#  puts "in InitConfig"
  global glob config env argv tcl_platform
  setDefaultConfig
  set glob(df) df
  set glob(searchstring) ""
  set glob(log) ""
  #set glob(left,pwd)  [frPwd]
  #set glob(right,pwd) [frPwd]
  set glob(history) ""
  foreach inst {left right} {
    set glob($inst,update_oldpwd) {} 
    set glob($inst,newpwd_oldpwd) {}
    set glob($inst,filelist) {}
    set glob($inst,pwd) {}
    set glob($inst,df) ?
    set glob($inst,dirstack) ""
    set glob($inst,shell,packed) 0
    set glob($inst,shell,history) {}
    set glob($inst,shell,complete,flipping) 0
    set glob($inst,shell,history,flipping) 0
    set glob($inst,shell,history,flipping,index) 0
    set glob($inst,shellcount) 0
    set glob(listbox,$inst) {}
  }
  set glob(abortcmd) 0
  set glob(toplevelidx) 0
  set glob(entry_dialog_return) {}
  set glob(ftp_entry_dialog_return) {}
  set glob(ftp,serverport,start) 5000
  set glob(ftp,serverport,end) 65000
  # This is an ugly hack to avoid reusing ports too fast, 
  # but it's because of a limitiation in the tcl socket
  # command. (You can't make the socket -server command 
  # choose a port by itself...)
  set glob(ftp,serverport,last) [expr 5000 + (([pid] * 100) % (65000 - 5000))]
  set glob(ftp,serversocket,acceptlist) {}
  set glob(forceupdate) 0
  set glob(havedoneftp) 0
  set glob(enableautoupdate) 1
  set glob(shell,maxed) 0
  set glob(batchlist) {}
  set glob(tmpdir) $glob(conf_dir)/tmp
  if {![file isdirectory $glob(tmpdir)]} {
    if {![file exists $glob(tmpdir)]} {
      set r [Try { file mkdir $glob(tmpdir) } "" 1]
      if {$r != 0} {
	PopError [_ "Can't create %s. May be problems later" $glob(tmpdir)]
      }
    }
  }	
  set glob(win,top) .fupper.ftop
  set glob(win,left) .fupper.fleft
  set glob(win,right) .fupper.fright
  set glob(win,middle) .fupper.can.fmiddle
  set glob(win,bottom) .flower

  set glob(gui,color_xx,winlist) ""
  set glob(gui,color_cmd,winlist) ""
  set glob(gui,color_scheme,winlist) ""
  set glob(gui,tablist) ""

  # These configuration have not been set yet, will be done in ReConfigColors
  set glob(gui,color_bg)  ""
  set glob(gui,color_fg)   ""
  set glob(gui,color_cmd)  ""
  set glob(gui,color_select_bg) ""
  set glob(gui,color_select_fg)  ""
  set glob(gui,color_highlight_bg) ""
  set glob(gui,color_highlight_fg) ""
  set glob(gui,color_balloonHelp_fg) ""
  set glob(gui,color_balloonHelp_bg) ""
  set glob(gui,color_scheme) ""
  set glob(gui,GuiFont) ""
  set glob(gui,color_cursor) ""
  set glob(gui,color_handle) ""

  set glob(gui,ListBoxFont) $config(gui,ListBoxFont)
  set glob(gui,GuiFont) ""
  set glob(gui,BalloonHelpFont)  $config(gui,BalloonHelpFont)  
  set glob(async) 0

  foreach ev {USER USERNAME LOGNAME} {
    if {![info exists env($ev)]} { 
      continue 
    }
    if {"[string range $env($ev) 0 0][string range $env($ev) end end]" == \
	    "%%" } {
      if {[string equal -nocase $ev "USER" ]} {
	unset env(USER)
      }
      continue
    }
    set env(USER) $env($ev)
    break
  }
  if {![info exists env(USER)]} { 
    if {![info exists env(HOME)]} {
      PopInfo [_ "Cannot read environment variables\
                  USER or USERNAME or LOGNAME or HOME.\
                  \nFileRunner cannot work without HOME and \
                  \n one of USER or USERNAME or LOGNAME correctly set."]
      CleanUp 1
    } else {
      set env(USER) [file tail $env(HOME)]
      PopInfo [_ "Environment variables 'USER' ane 'USERNAME' are undefined.\
                 \nDefaulting to $env(USER).\
                 \nTo stop this error report each time you\
                  run filerunner define environment variable\
                  'USER' or 'USERNAME'."]
    }
  }
  
  if {![info exists env(HOST)]} {
    if {![info exist env(COMPUTERNAME)] } {
      set env(HOST) unknown
    } else {
      set env(HOST) $env(COMPUTERNAME)
    }
  }
  if {$env(HOST) == "unknown"} {
    set r [catch { exec uname -n } out]
    if {!$r} {
      set env(HOST) $out
    }
  }

  # read history from disk
  if { [file exists $glob(conf_dir)/history ] } {
    set r [catch {set fid [open $glob(conf_dir)/history r]
      gets $fid glob(history)
      close $fid} out]
    if {$r} {
      puts [_ "FileRunner: Can't read directory history from disk: $out"]
    }
  }

  # set up a few default options
  #  option add *HighlightThickness 1 widgetDefault
  option add *InsertWidth 2 widgetDefault
  option add *Scrollbar.Width 11 widgetDefault
  option add *Button.BorderWidth 1 widgetDefault
  option add *Scrollbar.BorderWidth 2 widgetDefault
  #  option add *Listbox.BorderWidth 2 widgetDefault 
  # makes the spacing between list elements too large...
  option add *Frame.BorderWidth 2 widgetDefault
  option add *Entry.BorderWidth 2 widgetDefault
  #  option add *.Text.InsertBackground red widgetDefault
  #  option add *insertBorderWidth 2 widgetDefault
  #  option add *BorderWidth 1 widgetDefault
}

proc ReadConfig {{re 0}} {
  global glob glob config env argv argv0 tcl_platform
  set glob(select_pry_lr) {}
  # This gets rid of old menus and bindings.  
  #   array unset config menu,*
  #   array unset config bind,*
  lassign [getOldNewVersions] oldv newv

  # Here is where we set up a seperate config file for instances where fr is run
  # from a link with a different name. In such cases we name the config file
  # config-$name-revdate. We except common variants... 

  set runName [file tail $argv0]
  foreach exception [list \
			 tclsh* \
			 wish* \
			 fr.tcl \
			 fr-$newv-32.exe \
			 fr-$newv-64.exe \
			] {
    if {[string match $exception $runName]} {
      set runName "fr"
      break
    }
  }
  
  set oRoot $glob(configFileRoot)
  if {$runName != "fr" && ![string match "*-$runName" $glob(configFileRoot)]} {
    set glob(configFileRoot) $glob(configFileRoot)-$runName
  }
  
  set desired $glob(configFileRoot)-$newv
  if {![file readable $desired] } {
    Log "Config file $desired not found & readable."
    set m \[0-9\]
    cd $glob(conf_dir)
    # This should find all our old (and newer) configuration files...
    foreach filePat [list $glob(configFileRoot) $oRoot] {
      set pos \
	  [lsort -decreasing \
	       [glob -nocomplain -types {f r w} \
		    [file tail ${filePat}]-\[1-2\]$m$m$m$m$m ]]
      # on a new system or user there will be NO file...
      set from {}
      foreach try $pos {
	# here we insure that what we use is from an older rev and not a newer
	set rv [string range $glob(conf_dir)/$try end-5 end]
	# puts "$glob(conf_dir)/$try >$rv >$newv"
	if {[string compare $rv $newv] != 1} {
	  set from $glob(conf_dir)/$try
	  break
	}
      }
      if {$from != {} || $glob(configFileRoot) == $oRoot} {
	break
      }
    }
    
    if {$from == {}} {
      # if all that failed, check for the old name without the rev
      set from $glob(configFileRoot)/config
    }
    if {[file readable $from]} {
      Log "Found Config file $from. Moving to $desired"
      catch {file copy $from $desired}
    }
  }
  set glob(configFile) $desired
  if {[file readable $desired]} {
    #     puts "sourcing $desired"
    Log "Reading Configuration from $desired"
    setDefaultConfig
    set r [catch {source $desired} out]
    if {$r} {
      puts "$out [winfo exists .error_dialog]"
      smart_dialog .error_dialog "." [_ "Init error"] \
	  [list [_ "Reading user configuration\
           %s:\n%s" $desired  $out]] \
	  0 1 "OK" {maxw 16000}
    }
  }
  lappend ::startTimes [list [clock milliseconds] "After read config file"]
  if {$glob(cygwin) != {}} {
    # make sure df is properly named.
    set ops [FilenameComplete df 1]
    if {[lindex $ops 0] == {}} {
      # found df exact
      set glob(df) df
    } else {
      set idx [lsearch -glob $ops ".exe*"]
      if {$idx != -1} {
	set glob(df) df[lindex $ops $idx]
      } else {
	set glob(df) {}
      }
    }
  }
  set ::spSTRING $config(space)
  # We are about to re do the dir listing windows
  # First we must prevent the inotify code from updating while
  # The window isn't
  foreach inst {left right} {
    if {[info exists glob(inotify_after,$inst)] && \
	    $glob(inotify_after,$inst) != {}} {
      catch {after cancel $glob(inotify_after,$inst)}
    }
    set glob(inotify_after,$inst) "no"
    ClearCherryPicker $inst
  }
  catch {destroy $glob(win,left).frame_listb}
  catch {destroy $glob(win,right).frame_listb}
  if {$config(ftp,password) == "" } {
    set r [catch {set config(ftp,password) "$env(USER)@[exec domainname]"}]
  }
  # force a rebuild of the runlist
  if {[info exists glob(runMenu)]} {
    destroy $glob(runMenu)
  }
  # encript any open text passwords
  FixPasswords
  FixHotList
  FixFileNative
  # build the listbox script array
  foreach val $config(ListBoxScripts) {
    lassign $val name script
    set glob(lbscript,$name) $script
  }
  foreach k { left right } {
    #    puts "$config(ListBoxColumns,$k)"
    foreach ent $config(ListBoxColumns,$k) {
      lappend glob(listboxNames) [lindex $ent 0]
    }
  }
  #  puts "$glob(listboxNames)"
  set glob(listboxNames) [lsort -unique $glob(listboxNames)]
  foreach inst {left right} {
    foreach lbname $glob(listboxNames) {
      set glob($inst,lv$lbname) {}
    }
    if { !$glob(init_done)} {
      # A safe initial directory to get things started
 	NewPwd $inst ~/
    }
  }
  if {$glob(notify,Available)} {
    set glob(inotify_flags) $config(inotify_flags)
  }
  if { $config(ftp,proxy) == "" } {
    set config(ftp,useproxy) 0
  }
  if { [info exists config(focusFollowsMouse)] == 1 } {
    if { $config(focusFollowsMouse) != 0 } {
      tk_focusFollowsMouse
    }
  }
  if {$tcl_platform(platform) == "windows" } {
    set config(create_relative_links) 0
  }
  if { $config(dateformat) == "ddmmyy" } {
    GetTimeFromSecsSetFormat 1
  } elseif { $config(dateformat) == "yymmdd" } {
    GetTimeFromSecsSetFormat 0
  } else {
    GetTimeFromSecsSetFormat  $config(dateformat)
  }
  set ::balloon_help::enable $config(balloonhelp)
  lappend ::startTimes [list [clock milliseconds] "After post config show"]
  set deletedFCB [postConfigShow]
  BuildListBoxes
  # Really don't know why this needs to be called again, but if not
  # the center buttons aren't centered and the button scroll is off
  # especially with larger gui fonts
  postConfigShow
  lappend ::startTimes [list [clock milliseconds] "After post config list box build"]
  #  ReConfigColors
  #  ReConfigFont
  InitWindows
  lappend ::startTimes [list [clock milliseconds] "After post config init"]
  ForceUpdate
  # Allow the inotify code to do its thing again
  foreach inst {left right} {
    set glob(inotify_after,$inst) {}
  }
  lappend ::startTimes [list [clock milliseconds] "After post config update"]
  InitBindings
  if {$config(paste) != {}} {
    event add <<Paste>> {*}$config(paste)
  }
  # Here we do global bindings (config(global-bind,*))
  # First we deleate the old bindings...
  foreach {but val} [array get glob "bind,*,glob"] {
    set button [regsub {bind,(.*),glob} $but {\1}]
    catch {bind . <$button> {} }
  }
  # Now the new ones
  foreach {but val} [array get config "global-bind,*"] {
    set button [regsub {global-bind,(.*)} $but {\1}]
    catch {bind . <$button> {} }
    set glob(bind,$button,glob) [findCommand $val left]
    if {[catch {
      bind . <$button> "DoBut $button glob 0 %X %Y"} \
	     out] != 0 
      } {
      PopError [_ "In trying to do a global bind of '%s' the \
                \nerror '%s' occured. \
                 \n Skipping this binding." $button $out]
    }
  }
  if {[llength $deletedFCB] != 1} {
    lappend deletedFCB " \n" " \n"
    PopWarn [join $deletedFCB \n]
  }
  # Lastly, we allow configuration of the 3ed entry in the utilities menu.
  # The command is fine, but the label could have been changed...
  $glob(win,top).menu_frame.utils_but.m entryconfigure 3 \
      -label [lindex $config(cmd,ucmd) 0]

}
proc FixPasswords {} {
  FixThesePasswords config(ftp,login) {1 1}
  FixThesePasswords config(rsync,table) {4}
}
proc FixThesePasswords {location indx} {
  global config env
  # encript any open text passwords
  set new {}
  set i 0
  foreach login [set $location] {
    if {[lindex $login {*}$indx] == {} || \
	    ([lindex $login {*}$indx 0] == ">>" &&\
	    [lindex $login {*}$indx end] == "<<") } {
      #set passwd [lindex $login {*}$indx]
    } else {
      lset $location $i {*}$indx [concat ">>" \
		      [encript $env(USER) [lindex $login {*}$indx]] "<<" ]
    }
    # set tmp [list [lindex $login 0] [list [lindex $login 1 0] $passwd]]
    # if {[lindex $login 2] != "" } { lappend tmp [lindex $login 2]}    
    
    # lappend new $tmp
    incr i
  }
  # set config(ftp,login) $new
}

# We no longer want or need [file native construct] in the configuration file
# Here we carefully remove any that we find. We require:
#  "[<0-n spaces>file<1-n spaces>na<0-n of anything but space> <0-n of anything but ]>]"

proc FixFileNative {} {
  global config
  set exp {\[\s*file\s+na[^\s]*( *[^\]]*)\]}
  foreach {ent value} [array get config] {
    if {[regsub -all $exp $value {\1} value]} {
      set config($ent) $value
    }
  }
}

# We fix the old "-" prefix entries...
proc FixHotList {} {
  global config
  set new {}
  foreach ent $config(hotlist) {
    if {[string index [lindex $ent 0] 0] == "-" && [lassign $ent label val] != {}} {
      lappend new [list $label [list {*}[lrange $ent 1 end]]]
    } else {
      lappend new $ent
    }
  }
  set config(hotlist) $new
}

proc ConfigPwd {} {
  global glob config env argv argv0
  # Normalize these just in case they are MS windows native names...
  set pwd(left) [file norm [lindex $argv 0]]
  set pwd(right) [file norm [lindex $argv 1]]
  foreach k { left right } {
    # allow various perversions on what we find...
    if {![file exists $pwd($k)] && $pwd($k) != {}} {
      # not a file or dir, does it lead with a dir?
      set pwd($k) [file dirname $pwd($k)]
    }
    if {![file exists $pwd($k)]} {
      # still not a file or dir, try the config name
      set pwd($k) $config(startpwd,$k)
    }
    if {![file exists $pwd($k)]} {
      # every thing failed, forget about it.
      #continue
      set pwd($k) $glob(start_path)
    }
    if {[file isdirectory $pwd($k)]} {
      # if it is a dir, just go there
      NewPwd $k $pwd($k)
      UpdateWindow $k  
    } else {
      # if it is a full file name, go to that dir and highlight the file
      GotoFind $pwd($k) $pwd($k) $k
    }
  }
  AppendToDirHistory $glob(left,pwd)
  AppendToDirHistory $glob(right,pwd)
  set glob(left,dirstack) $glob(left,pwd)
  set glob(right,dirstack) $glob(right,pwd)
}

proc CheckConfigDir {} {
  global env glob

  if {! [file isdirectory $glob(conf_dir)]} {

    set r [catch [file mkdir $glob(conf_dir)] out]
    if {$r} {
      regsub -all {\\} $out {/} out
      PopErrorSimple "$out"
      exit 1
    }
  }
  set glob(configFileRoot) $glob(conf_dir)/config$glob(cygwin)
}

proc ConfigBrowser {} {
  global glob config
  lassign [getOldNewVersions] oldv newv

  if {![file exists $glob(configFile)]} { SaveConfig }

  set glob(edit_configuration_configlist) {
    {Hotlists {HOTLISTS}}
    hotlist
    runlist
    {{Mouse Bindings & menus} {MOUSE & KEY BINDINGS & MENUS}}
    {{  bind} {Each entry binds}}
    {{  menu} {Mail To}}
    {{  exception} no_selection}
    paste
      {Commands {COMMAND CONFIGURATION}}
    cmd,diff
    cmd,open
    cmd,print
    cmd,term
    editor
    open,extensions
    space
    view,extensions
      {Archiver/Packer {ARCHIVER/PACKER}}
    cmd,archive
    cmd,pack
    cmd,unarc,extensions
    cmd,unpack,extensions
    {FTP {FTP CONFIGURATION}}
    ftp,anonymous
    ftp,cache,maxentries
    ftp,cd_pwd
    ftp,login
    ftp,fastlink
    ftp,password
    ftp,proxy
    ftp,timeout
    ftp,useproxy
      {RSYNC {RSYNC CONFIGURATION}}
    rsync,table
      {HTTP {HTTP CONFIGURATION}}
    http,proxy
      {Appearance {APPEARANCE}}
    columnScroll
    columnScrollSize
    geometry,main
    geometry,textviewer
    geometry,qedit
    gray
    gui,color_bg
    gui,color_fg
    gui,color_cmd
    gui,color_select_bg
    gui,color_select_fg
    gui,color_highlight_bg
    gui,color_highlight_fg
    gui,color_handle
    gui,color_scheme
    gui,color_cursor
    gui,color_flash
    gui,color_balloonHelp_fg
    gui,color_balloonHelp_bg
    gui,ListBoxFont
    gui,GuiFont
    gui,BalloonHelpFont
    ListBoxColumns,right
    ListBoxColumns,left
    ListBoxScripts
    middle_button_list
    middle_button_colors
      {Shell {SHELL}}
    shell,aliases
    shell,buffer
    shell,height,left
    shell,height,right
      {Miscellaneous {MISC}}
    ask,dir_delete
    ask,file_delete
    autoupdate
    balloonhelp
    cd_pwd
    check_ownership
    create_relative_links
    dateformat
    fast_checkboxes
    fileshow,all
    fileshow,dirs
    fileshow,sort
    focusFollowsMouse
    [expr {$::MSW ? "" : "inotify_flags"}]
    keyb_support
    logsize
    mwheel,delta
    mwheel,neg
    mwheel,pos
    save_conf_at_exit
    spellingFilter
    startpwd,left
    startpwd,right
      {{Dangerous stuff} {DANGEROUS}}
    [expr {$::MSW ? "" : "cmd,chmod"}]
    cmd,cp
    cmd,dircp
    cmd,du
    cmd,find
    cmd,hardlink
    cmd,rm
    cmd,sh
    cmd,tail
    cmd,touch
    cmd,ucmd
    [expr {$::MSW ? "useDIR" : ""}]
  }
  # now do the substutions for windows vs unix
  set glob(edit_configuration_configlist) \
      [subst  $glob(edit_configuration_configlist)]

  set w .config_dialog
  if {[winfo exists $w]} {
    PopWarn "Configuration window already open!"
    return
  }
  toplevel $w -class Dialog
  wm title $w "Configuration Browser"
  wm iconname $w "FR config browser"
  wm resizable $w true true
  wm transient $w [winfo toplevel [winfo parent $w]]

  frame $w.left
  frame $w.right
  frame $w.bot
  
  # Create left frame containing the list of config options
  scrollbar $w.left.scrollvert -command "$w.left.list yview" -orient vertical 
  listbox $w.left.list \
      -yscrollcommand "$w.left.scrollvert set" \
      -font $config(gui,ListBoxFont) \
      -width 25 \
      -height 35 
      # -background $config(gui,color_bg)\
      # -foreground $config(gui,color_fg) \
      # -selectbackground $config(gui,color_select_bg)\
      # -selectforeground $config(gui,color_select_fg) 
  pack $w.left.list -side left -fill both
  pack $w.left.scrollvert -side right -fill y

  # Create right frame containing the config text and a scrollbar
  text $w.right.text\
      -relief sunken \
      -bd 2 \
      -yscrollcommand "$w.right.scroll set" \
      -setgrid 1 \
      -height 35 \
      -undo 1 \
      -font $config(gui,ListBoxFont)\
      -highlightthickness 0 
      # -background $config(gui,color_bg) \
      # -foreground $config(gui,color_fg) \
      # -selectbackground $config(gui,color_select_bg) \
      # -selectforeground $config(gui,color_select_fg) 
  scrollbar $w.right.scroll -command "$w.right.text yview" 
  pack $w.right.scroll -side right -fill y
  pack $w.right.text -expand 1 -fill both

  # Create bottom frame containing the OK/Cancel buttons
  button $w.bot.regen \
      -text {Regenerate Configuration}  \
      -command "ConfigBrowserRegenerate $w.right.text"
  button $w.bot.ok -text OK -command "ConfigBrowserOK $w.right.text $w"
  button $w.bot.cancel \
      -text Cancel  \
      -command "set glob(config_dialog_return) {}; destroy $w"
  pack $w.bot.cancel -side right -padx 3 -fill x
  pack $w.bot.ok -side right -padx 3
  pack $w.bot.regen -side right -padx 3 -fill x

  # Fill left frame with contents
  #set glob(font_dialog,fl) [split $glob(font_dialog,fl) "\n"]
  set cdl {}
  foreach k $glob(edit_configuration_configlist) {
    if {[lindex $k 1] != {}} {
      lappend cdl [lindex $k 0]
    } else {
      set tmp [split $k ,_]
      set tmp2 {}
      foreach t $tmp {
        lappend tmp2 "[string toupper [string index $t 0]][string range $t 1 end]"
      }
      lappend cdl "  $tmp2"
    }
  }

  eval $w.left.list insert end $cdl

  # Create popup menu
  textSearch $w.right.text [_ "Cfg edit"] "+buildViewConfig" {} \
      [list Undo "undoHelp $w.right.text undo" \
	   Redo  "undoHelp $w.right.text redo"\
	   {Test current} "ConfigTest \[$w.right.text index insert]" \
	   {Test All}    "ConfigTest All" \
	  ]    
  
  pack $w.bot -side bottom
  pack $w.left -side left -fill both
  pack $w.right -side right -fill both -expand 1

  # Assign bindings
  bind $w.right.text $config(mwheel,neg) \
      "$w.right.text yview scroll -$config(mwheel,delta) units;break"
  bind $w.right.text $config(mwheel,pos) \
      "$w.right.text yview scroll $config(mwheel,delta) units;break"
  bind $w.left.list $config(mwheel,neg) \
      "$w.left.list yview scroll -$config(mwheel,delta) units;break"
  bind $w.left.list $config(mwheel,pos) \
      "$w.left.list yview scroll $config(mwheel,delta) units;break"
  bind $w.left.list <1> \
      "ConfigBrowserJumpToLoc $w.right.text \[$w.left.list nearest %y\]"
  wm withdraw $w
  update idletasks
  centerWin $w
  wm deiconify $w

  # Fill right frame with contents
  set filename $glob(configFile)
  set r [catch {set fid [open $filename r]} out]
  if {$r} {
    PopError "Can't open $filename for editing"
    destroy $w
    return
  }
  $w.right.text insert 0.0 [read -nonewline $fid]
  $w.right.text edit reset
  close $fid
  $w.right.text mark set insert 0.0
}

# The idea here is to test the statement to see if it is error free
# Either "All" or at insert mark.  It is an error if the insert mark
# is not on a line with a "set" in it.

proc ConfigTest {what args} {
  set w .config_dialog.right.text
  # we keep config as a local var so the result is NOT written to the global
  lassign [split [$w index end] .] max ch
  if {$what != "All"} {
    lassign [split $what .] line char
  } else {
    set line 1
  }
  set end [expr {$line + 1}].0
  set text [$w get $line.0 $end]
  if {$what != "All" &&\
	  ([string index [string trim $line] 0] == "#" || \
	       ![string match {set *} $text] ) } {
    $w tag add sel $line.0 $end
    PPopWarn "Line does not start with \"set \""
    return
  }
  # Ok, here we will accumulate lines until we find another "set" or "#"
  while {$line < $max} {
    while {$line < $max && \
	     ([string index [string trim [set text [$w get $line.0 $end]]] 0] == "#" ||\
		  ![string match {set *} $text])} {
      # this is comment or white space
      incr line
      set end [expr {$line + 1}].0
   }
    # eof?
    $w tag add sel $line.0 $end
    set startLine $line
    incr line
    if {$line >= $max} {
      if {$args == {}} {
	PPopWarn [_ "Looks good."]
      }
      return 1
    }
    set stmt $text
    while {$line < $max && \
	       [string index \
		    [string trim \
			 [set text [$w get $line.0 [expr $line + 1].0]]] 0] != "#" && \
	       ! [string match {set *} $text] && \
	       [string trim $text] != {}} {
      append stmt $text
      incr line
      
    }
    #$w tag remove sel 0.0 end
    $w tag add sel $startLine.0 $line.0
    set r [catch "eval {$stmt}"]
    # puts "Testing: $stmt"
    set mid [expr {($startLine + $line) / 2}].0
    if {$r != 0} {
      $w see 1.0
      $w see $mid
      PopWarn "$::errorInfo"
      return 0
    }
    if {$what != "All"} {
      $w see 1.0
      $w see $mid
      PPopWarn [_ "Looks good."]
      return 
    }
    $w tag remove sel 0.0 end
  }
  PPopWarn [_ "Looks good."]
  return 1
  
}

# pop window up over the left side of the config window
proc PPopWarn {info} {
  smart_dialog .apop .config_dialog.left [_ "Info"] [list $info] 0 1 [_ "OK"]

}

proc ConfigBrowserJumpToLoc { w index } {
  global glob config
  set tmp [lindex $glob(edit_configuration_configlist) $index]
  if {[lindex $tmp 1] != {}} {
    set searchfor [lindex $tmp 1]
    set heading 1
  } else {
    set searchfor config($tmp)
    set heading 0
  }

  set tag select
  $w tag configure $tag\
      -background $config(gui,color_select_bg)\
      -foreground $config(gui,color_select_fg) 
  $w tag remove $tag 0.0 end

  set idx [$w search -count len -backwards -- $searchfor 0.0]
  if {$idx == ""} {
    PopWarn "Can't find that location in config file. \
      Try to regenerate configuration and try again."
    return
  }
  $w mark set insert "$idx + $len chars"
  $w see end
  $w see insert
  #  puts "$idx $len"
  set line [lindex [split $idx .] 0]
  if {$heading} {
    $w mark set insert $line.0
  } else {
    # search backwards for ------------ and make sure it's visible
    set idx [$w search -count len -backwards -- "-----------" insert]
    if {$idx != {}} {
      $w see $idx
      # The above may have pushed the object out.  Put it back.
      $w see insert
    }
  }
  $w tag add $tag $line.0 [expr $line+1].0
}

proc ConfigBrowserOK { w wm} {
  if {![ConfigTest All No]} {return}
  global glob config
  lassign [getOldNewVersions] oldv newv
  set filename $glob(configFile)
  Log "Configuration Browser: Saving $filename"
  Try { set fid [open $filename w]
    puts -nonewline $fid [$w get 0.0 end]
    close $fid} "" 1
  # No more need for window
  destroy $wm
  catch {destroy $glob(win,left).frame_listb}
  catch {destroy $glob(win,right).frame_listb}
  set glob(select_pry_lr) {}
  # This gets rid of old menus and bindings.  
  array unset config menu,*
  array unset config bind,*
  ReadConfig
  #ForceUpdate
}

proc ConfigBrowserRegenerate { w } {
  global glob
  lassign [getOldNewVersions] oldv newv
  SaveConfig
  set filename $glob(configFile)
  set r [catch {set fid [open $filename r]} out]
  if {$r} {
    PopError "Can't open $filename for editing"
    return
  }
  $w delete 0.0 end
  $w insert 0.0 [read -nonewline $fid]
  close $fid
  $w mark set insert 0.0
}
