#
# A short function to build menus from structures
#
# <20161222.1757.14>
#
# This function takes a complex structure and builds a menu from it.
#
# The call:
#
# buildCasMenu struct1 struct2 button action ?options?
#
# struct1 is is the head info for the menu. It may be empty, but
#         if not, is intended to contain entries the are always
#         the same. Entries may be:
#         {}          empty, creates a seperator
#         simple      creates an entry with label 'simple' returns 'simple'
#         {lab sm}    creates an entry with label 'lab', returns 'sm'
#         {lab {new}} where 'new' is a sub list of at least two entries
#                     creats a cascade with label 'lab'. Each entry in
#                     'new' creates an entry in the sub menu.
#                     this may be continued for as may cascades as needed.
#         If struct1 is {} any existing menu entries will not be cleared.

# struct2 is the structure containing the menu items. This has the same
#         format as struct1 (infact it is concationated with struct1)

# button is the menu is to be built. This should exist
#        when the call is made.  Any menu entries on this button will be
#        cleared when the call is made (to allow dynamic menus)
# action is a function that is the command that is called when a menu
#        item is selected. It will get three parameters:
#        A) the 'returns' value (see above list structure) and
#        b) 1 if from struct1 and 2 if from struc2
#        C) the entry number which is the lindex key for the entry
#           For cascade menus this will have form {n m ...}
#        Each 
# options is an optional list of dublets of the form:
#         {-ent value} which are to be used with the menu. Most useful
#         here is "-tearoffcommand xxx" and "-tearoff true -font 'font'"
#         These entries are only used on cascade menus.
#
proc buildCasMenu {s1 s2 w ac {ops {}}} {
  # our master window
  set wh 1
  if {$s1 != {}} {
    $w delete 0 end
  }
  foreach list [list $s1 $s2] {
    buildMenuWork $w $list {}
    incr wh
  }  
}


proc buildMenuWork {w li sub} {
  upvar ac ac wh wh ac ac ops ops
  set level -1
  foreach en $li {
    incr level
    set tag [expr {$sub == {} ? $level : [list {*}$sub $level]}]
    set remains [lassign $en lab val]
    # puts "label $lab"
     switch -glob $lab  {
      {} {$w add separator}
      -* {
        # puts "$w $level $en $lab $val"
	destroy $w.$level
	menu $w.$level -title [string range $lab 1 end]  {*}$ops
	$w add cascade -menu $w.$level -label [string range $lab 1 end]
	$w.$level delete 0 end
	buildMenuWork $w.$level $val [list {*}$tag 1]
      }
      default {
	if {$remains != {}} {
	  PopWarn [_ "Ill formed list at \"%s\"" $en]
	}
	if {$val == {}} {
	  set val $lab
	}
	$w add command -label "$lab" -command [list {*}$ac $val $wh   $tag]}
    }
  }
}
