# <20170330.1552.13>
# This function is to set up a enhanced 'puts' routine (see below)
# which can be globally turned off...
#
#  setupDebug   defines either a dummy or the real function
#
#  parameter  on  1 turns on the debug routine
#                 0 (or {}) makes the debug routine a dummy
#

# This is here to cause auto_mkindex to index us.

proc frputs {args} {}

set frputsOps {}

proc frputsOps {args} {
  set ::frputsOps $args
}

proc setupDebug {{on 0} } {
  if {$on} {
    # A smart puts routine.
    # First the name of the calling 'proc' and its line number in its
    # source file is put in the output line, i.e.
    # '<proc name> <(line#)>: '
    # If ::frputOps is not {} then it is taken as a set of return ops for info frame
    # and each one, if it exists is put in the ouput line
    # Then each argument is tested for a trailing blank
    # if there is one, the argument is put in the final string
    # if not, the argument is assumed to be a var name.  If it exists:
    # '<name>=$<name>< ' is put int he final string, if not '<unset> ' is used as 
    #  the value. Finally, the final string is processed to change all
    #  \r \t and \n chars to the backslash equivalents.

    proc frputs { args } {
      set m {}
      array unset info
      array set info [info frame -1]
      if {[info exists info(proc)]} {
	append m "$info(proc)"
      }
      if {[info exists info(line)]} {
	append m " ($info(line))"
      }
      foreach en $::frputsOps {
	append m " $en="
	if {[info exists info($en)]} {
	  append m "[set info($en)]"
	} else {
	  append m "<unset>"
	}
	append m "< "
      }
      append m ": "
      foreach ar  $args {
	if {[string index $ar end] == " " } {
	  append m [string range $ar 0 end]
	} elseif { ! [catch "uplevel \"info exists $ar\" " ro] &&  $ro } {
	  append m "$ar=[uplevel "set $ar"]< "
	} else {
	  append m "$ar=<unset>< "
	}
      }
      regsub -all {\n} $m {\\n} m
      regsub -all {\r} $m {\\r} m
      regsub -all {\t} $m {\\t} m
      puts "[set m]"
      flush stdout
    }
  } else {
    proc frputs {args} {}
  }
}
# So you don't need to call setupDebug...
setupDebug

# here is a little thing to call from tkcon to print lists
# It puts 'co' elements on a line and outputs the result.
proc lh {list {co {1}}} {
  set idx 0
  while {$idx < [llength $list]} {
    set li {}
    for {set x $co} {$x > 0} {incr x -1; incr idx} {
      lappend li [lindex $list $idx]
    }     
    puts "$li"
  }
}
